# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\parameter_configuration.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ConfigurationDialog(object):
    def setupUi(self, ConfigurationDialog):
        ConfigurationDialog.setObjectName("ConfigurationDialog")
        ConfigurationDialog.resize(1054, 608)
        font = QtGui.QFont()
        font.setPointSize(10)
        ConfigurationDialog.setFont(font)
        self.GridLayout = QtWidgets.QGridLayout(ConfigurationDialog)
        self.GridLayout.setVerticalSpacing(22)
        self.GridLayout.setObjectName("GridLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.GridLayout.addItem(spacerItem, 2, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(ConfigurationDialog)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.GridLayout.addWidget(self.buttonBox, 2, 2, 1, 2)
        self.restore_standard_values = QtWidgets.QPushButton(ConfigurationDialog)
        self.restore_standard_values.setObjectName("restore_standard_values")
        self.GridLayout.addWidget(self.restore_standard_values, 2, 1, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(ConfigurationDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setMinimumSize(QtCore.QSize(700, 0))
        self.tabWidget.setObjectName("tabWidget")
        self.tab_workflow = QtWidgets.QWidget()
        self.tab_workflow.setObjectName("tab_workflow")
        self.gridLayout = QtWidgets.QGridLayout(self.tab_workflow)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 11, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem2, 0, 1, 1, 1)
        self.nfs_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.nfs_checkBox.setObjectName("nfs_checkBox")
        self.gridLayout.addWidget(self.nfs_checkBox, 7, 5, 1, 2)
        self.mr_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.mr_checkBox.setObjectName("mr_checkBox")
        self.gridLayout.addWidget(self.mr_checkBox, 16, 1, 1, 1)
        self.spp_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.spp_checkBox.setObjectName("spp_checkBox")
        self.gridLayout.addWidget(self.spp_checkBox, 18, 1, 1, 1)
        self.gppl_spinBox = QtWidgets.QSpinBox(self.tab_workflow)
        self.gppl_spinBox.setMaximum(2)
        self.gppl_spinBox.setProperty("value", 1)
        self.gppl_spinBox.setObjectName("gppl_spinBox")
        self.gridLayout.addWidget(self.gppl_spinBox, 1, 6, 1, 1)
        self.pfs_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.pfs_checkBox.setObjectName("pfs_checkBox")
        self.gridLayout.addWidget(self.pfs_checkBox, 8, 5, 1, 2)
        self.mr_lineEdit = QtWidgets.QLineEdit(self.tab_workflow)
        self.mr_lineEdit.setObjectName("mr_lineEdit")
        self.gridLayout.addWidget(self.mr_lineEdit, 16, 2, 1, 1)
        self.gpif_comboBox = QtWidgets.QComboBox(self.tab_workflow)
        self.gpif_comboBox.setObjectName("gpif_comboBox")
        self.gridLayout.addWidget(self.gpif_comboBox, 3, 6, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem3, 19, 1, 1, 1)
        self.nap_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.nap_checkBox.setObjectName("nap_checkBox")
        self.gridLayout.addWidget(self.nap_checkBox, 10, 5, 1, 2)
        self.gpspwr_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.gpspwr_checkBox.setObjectName("gpspwr_checkBox")
        self.gridLayout.addWidget(self.gpspwr_checkBox, 3, 1, 1, 1)
        self.ipfn_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.ipfn_checkBox.setObjectName("ipfn_checkBox")
        self.gridLayout.addWidget(self.ipfn_checkBox, 7, 1, 1, 2)
        self.apbs_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.apbs_checkBox.setObjectName("apbs_checkBox")
        self.gridLayout.addWidget(self.apbs_checkBox, 9, 5, 1, 2)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem4, 17, 1, 1, 1)
        self.gpbl_combobox = QtWidgets.QComboBox(self.tab_workflow)
        self.gpbl_combobox.setObjectName("gpbl_combobox")
        self.gridLayout.addWidget(self.gpbl_combobox, 12, 2, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem5, 4, 1, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem6, 2, 1, 1, 1)
        self.gpbl_label_parameter = QtWidgets.QLabel(self.tab_workflow)
        self.gpbl_label_parameter.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.gpbl_label_parameter.setObjectName("gpbl_label_parameter")
        self.gridLayout.addWidget(self.gpbl_label_parameter, 12, 1, 1, 1)
        self.gppl_label_parameter = QtWidgets.QLabel(self.tab_workflow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.gppl_label_parameter.sizePolicy().hasHeightForWidth())
        self.gppl_label_parameter.setSizePolicy(sizePolicy)
        self.gppl_label_parameter.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.gppl_label_parameter.setObjectName("gppl_label_parameter")
        self.gridLayout.addWidget(self.gppl_label_parameter, 1, 5, 1, 1)
        self.gpif_label_parameter = QtWidgets.QLabel(self.tab_workflow)
        self.gpif_label_parameter.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.gpif_label_parameter.setObjectName("gpif_label_parameter")
        self.gridLayout.addWidget(self.gpif_label_parameter, 3, 5, 1, 1)
        self.gpwptf_checkBox = QtWidgets.QCheckBox(self.tab_workflow)
        self.gpwptf_checkBox.setChecked(True)
        self.gpwptf_checkBox.setObjectName("gpwptf_checkBox")
        self.gridLayout.addWidget(self.gpwptf_checkBox, 1, 1, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem7, 1, 7, 1, 1)
        self.mr_label = QtWidgets.QLabel(self.tab_workflow)
        self.mr_label.setText("")
        self.mr_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.mr_label.setWordWrap(True)
        self.mr_label.setObjectName("mr_label")
        self.gridLayout.addWidget(self.mr_label, 12, 4, 7, 4)
        self.gridLayout.setColumnStretch(3, 4)
        self.gridLayout.setColumnStretch(7, 2)
        self.tabWidget.addTab(self.tab_workflow, "")
        self.tab_frames = QtWidgets.QWidget()
        self.tab_frames.setObjectName("tab_frames")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_frames)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem8, 13, 0, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem9, 3, 0, 1, 1)
        self.efs_checkBox = QtWidgets.QCheckBox(self.tab_frames)
        self.efs_checkBox.setObjectName("efs_checkBox")
        self.gridLayout_2.addWidget(self.efs_checkBox, 14, 0, 1, 1)
        self.afm_comboBox = QtWidgets.QComboBox(self.tab_frames)
        self.afm_comboBox.setCurrentText("")
        self.afm_comboBox.setMaxVisibleItems(2)
        self.afm_comboBox.setObjectName("afm_comboBox")
        self.gridLayout_2.addWidget(self.afm_comboBox, 7, 2, 1, 2)
        self.afa_checkBox = QtWidgets.QCheckBox(self.tab_frames)
        self.afa_checkBox.setChecked(True)
        self.afa_checkBox.setObjectName("afa_checkBox")
        self.gridLayout_2.addWidget(self.afa_checkBox, 8, 0, 1, 1)
        self.afrsf_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.afrsf_label_parameter.setObjectName("afrsf_label_parameter")
        self.gridLayout_2.addWidget(self.afrsf_label_parameter, 9, 0, 1, 1)
        self.fgw_slider_value = QtWidgets.QSlider(self.tab_frames)
        self.fgw_slider_value.setMinimum(1)
        self.fgw_slider_value.setMaximum(6)
        self.fgw_slider_value.setSingleStep(1)
        self.fgw_slider_value.setPageStep(1)
        self.fgw_slider_value.setProperty("value", 4)
        self.fgw_slider_value.setSliderPosition(4)
        self.fgw_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.fgw_slider_value.setObjectName("fgw_slider_value")
        self.gridLayout_2.addWidget(self.fgw_slider_value, 4, 2, 1, 2)
        self.afafp_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.afafp_label_parameter.setObjectName("afafp_label_parameter")
        self.gridLayout_2.addWidget(self.afafp_label_parameter, 12, 0, 1, 1)
        self.fgw_label_display = QtWidgets.QLabel(self.tab_frames)
        self.fgw_label_display.setObjectName("fgw_label_display")
        self.gridLayout_2.addWidget(self.fgw_label_display, 4, 5, 1, 1)
        spacerItem10 = QtWidgets.QSpacerItem(36, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem10, 4, 4, 1, 1)
        self.afrsf_label_display = QtWidgets.QLabel(self.tab_frames)
        self.afrsf_label_display.setObjectName("afrsf_label_display")
        self.gridLayout_2.addWidget(self.afrsf_label_display, 9, 5, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem11, 11, 0, 1, 1)
        self.afafp_label_display = QtWidgets.QLabel(self.tab_frames)
        self.afafp_label_display.setObjectName("afafp_label_display")
        self.gridLayout_2.addWidget(self.afafp_label_display, 12, 5, 1, 1)
        self.afsw_label_display = QtWidgets.QLabel(self.tab_frames)
        self.afsw_label_display.setObjectName("afsw_label_display")
        self.gridLayout_2.addWidget(self.afsw_label_display, 10, 5, 1, 1)
        self.fgw_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.fgw_label_parameter.setObjectName("fgw_label_parameter")
        self.gridLayout_2.addWidget(self.fgw_label_parameter, 4, 0, 1, 1)
        self.afsw_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.afsw_label_parameter.setObjectName("afsw_label_parameter")
        self.gridLayout_2.addWidget(self.afsw_label_parameter, 10, 0, 1, 1)
        self.afafp_slider_value = QtWidgets.QSlider(self.tab_frames)
        self.afafp_slider_value.setMinimum(3)
        self.afafp_slider_value.setMaximum(30)
        self.afafp_slider_value.setSingleStep(3)
        self.afafp_slider_value.setPageStep(3)
        self.afafp_slider_value.setProperty("value", 6)
        self.afafp_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.afafp_slider_value.setObjectName("afafp_slider_value")
        self.gridLayout_2.addWidget(self.afafp_slider_value, 12, 2, 1, 2)
        spacerItem12 = QtWidgets.QSpacerItem(40, 25, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem12, 4, 1, 1, 1)
        spacerItem13 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem13, 5, 0, 2, 1)
        spacerItem14 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem14, 0, 0, 1, 1)
        self.fdb_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.fdb_label_parameter.setObjectName("fdb_label_parameter")
        self.gridLayout_2.addWidget(self.fdb_label_parameter, 1, 0, 1, 1)
        self.afrsf_slider_value = QtWidgets.QSlider(self.tab_frames)
        self.afrsf_slider_value.setMinimum(5)
        self.afrsf_slider_value.setMaximum(80)
        self.afrsf_slider_value.setSingleStep(5)
        self.afrsf_slider_value.setPageStep(5)
        self.afrsf_slider_value.setProperty("value", 25)
        self.afrsf_slider_value.setSliderPosition(25)
        self.afrsf_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.afrsf_slider_value.setObjectName("afrsf_slider_value")
        self.gridLayout_2.addWidget(self.afrsf_slider_value, 9, 2, 1, 2)
        self.afsw_slider_value = QtWidgets.QSlider(self.tab_frames)
        self.afsw_slider_value.setMinimum(5)
        self.afsw_slider_value.setMaximum(150)
        self.afsw_slider_value.setSingleStep(5)
        self.afsw_slider_value.setPageStep(5)
        self.afsw_slider_value.setProperty("value", 20)
        self.afsw_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.afsw_slider_value.setObjectName("afsw_slider_value")
        self.gridLayout_2.addWidget(self.afsw_slider_value, 10, 2, 1, 2)
        spacerItem15 = QtWidgets.QSpacerItem(271, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem15, 4, 6, 1, 1)
        self.afm_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.afm_label_parameter.setObjectName("afm_label_parameter")
        self.gridLayout_2.addWidget(self.afm_label_parameter, 7, 0, 1, 1)
        self.fco_checkBox = QtWidgets.QCheckBox(self.tab_frames)
        self.fco_checkBox.setObjectName("fco_checkBox")
        self.gridLayout_2.addWidget(self.fco_checkBox, 14, 2, 1, 4)
        self.fdbm_label_parameter = QtWidgets.QLabel(self.tab_frames)
        self.fdbm_label_parameter.setObjectName("fdbm_label_parameter")
        self.gridLayout_2.addWidget(self.fdbm_label_parameter, 2, 0, 1, 1)
        self.fdb_comboBox = QtWidgets.QComboBox(self.tab_frames)
        self.fdb_comboBox.setObjectName("fdb_comboBox")
        self.gridLayout_2.addWidget(self.fdb_comboBox, 1, 2, 1, 4)
        self.fdbm_comboBox = QtWidgets.QComboBox(self.tab_frames)
        self.fdbm_comboBox.setObjectName("fdbm_comboBox")
        self.gridLayout_2.addWidget(self.fdbm_comboBox, 2, 2, 1, 4)
        self.gridLayout_2.setColumnStretch(0, 3)
        self.gridLayout_2.setColumnStretch(1, 1)
        self.gridLayout_2.setColumnStretch(2, 2)
        self.gridLayout_2.setColumnStretch(3, 2)
        self.gridLayout_2.setColumnStretch(4, 1)
        self.gridLayout_2.setColumnStretch(5, 2)
        self.gridLayout_2.setColumnStretch(6, 7)
        self.gridLayout_2.setRowStretch(0, 3)
        self.gridLayout_2.setRowStretch(1, 3)
        self.gridLayout_2.setRowStretch(2, 3)
        self.gridLayout_2.setRowStretch(3, 3)
        self.gridLayout_2.setRowStretch(4, 3)
        self.gridLayout_2.setRowStretch(6, 3)
        self.gridLayout_2.setRowStretch(7, 3)
        self.gridLayout_2.setRowStretch(8, 3)
        self.gridLayout_2.setRowStretch(9, 3)
        self.gridLayout_2.setRowStretch(10, 3)
        self.gridLayout_2.setRowStretch(11, 3)
        self.gridLayout_2.setRowStretch(12, 3)
        self.tabWidget.addTab(self.tab_frames, "")
        self.tab_alignment = QtWidgets.QWidget()
        self.tab_alignment.setObjectName("tab_alignment")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_alignment)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.apsw_slider_value = QtWidgets.QSlider(self.tab_alignment)
        self.apsw_slider_value.setMinimum(6)
        self.apsw_slider_value.setMaximum(30)
        self.apsw_slider_value.setSingleStep(2)
        self.apsw_slider_value.setPageStep(2)
        self.apsw_slider_value.setProperty("value", 10)
        self.apsw_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.apsw_slider_value.setObjectName("apsw_slider_value")
        self.gridLayout_3.addWidget(self.apsw_slider_value, 3, 2, 1, 1)
        self.apst_label_parameter = QtWidgets.QLabel(self.tab_alignment)
        self.apst_label_parameter.setObjectName("apst_label_parameter")
        self.gridLayout_3.addWidget(self.apst_label_parameter, 5, 0, 1, 1)
        self.apst_slider_value = QtWidgets.QSlider(self.tab_alignment)
        self.apst_slider_value.setMinimum(1)
        self.apst_slider_value.setMaximum(30)
        self.apst_slider_value.setPageStep(5)
        self.apst_slider_value.setProperty("value", 5)
        self.apst_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.apst_slider_value.setObjectName("apst_slider_value")
        self.gridLayout_3.addWidget(self.apst_slider_value, 5, 2, 1, 1)
        self.apsw_label_display = QtWidgets.QLabel(self.tab_alignment)
        self.apsw_label_display.setObjectName("apsw_label_display")
        self.gridLayout_3.addWidget(self.apsw_label_display, 3, 4, 1, 1)
        self.apsw_label_parameter = QtWidgets.QLabel(self.tab_alignment)
        self.apsw_label_parameter.setObjectName("apsw_label_parameter")
        self.gridLayout_3.addWidget(self.apsw_label_parameter, 3, 0, 1, 1)
        self.apst_label_display = QtWidgets.QLabel(self.tab_alignment)
        self.apst_label_display.setObjectName("apst_label_display")
        self.gridLayout_3.addWidget(self.apst_label_display, 5, 4, 1, 1)
        self.aphbw_slider_value = QtWidgets.QSlider(self.tab_alignment)
        self.aphbw_slider_value.setMinimum(20)
        self.aphbw_slider_value.setMaximum(140)
        self.aphbw_slider_value.setSingleStep(4)
        self.aphbw_slider_value.setPageStep(4)
        self.aphbw_slider_value.setProperty("value", 40)
        self.aphbw_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.aphbw_slider_value.setObjectName("aphbw_slider_value")
        self.gridLayout_3.addWidget(self.aphbw_slider_value, 1, 2, 1, 1)
        self.aphbw_label_display = QtWidgets.QLabel(self.tab_alignment)
        self.aphbw_label_display.setObjectName("aphbw_label_display")
        self.gridLayout_3.addWidget(self.aphbw_label_display, 1, 4, 1, 1)
        spacerItem16 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem16, 8, 1, 1, 1)
        spacerItem17 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem17, 1, 1, 1, 1)
        spacerItem18 = QtWidgets.QSpacerItem(44, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem18, 1, 3, 1, 1)
        self.apbt_slider_value = QtWidgets.QSlider(self.tab_alignment)
        self.apbt_slider_value.setMinimum(2)
        self.apbt_slider_value.setMaximum(50)
        self.apbt_slider_value.setSingleStep(2)
        self.apbt_slider_value.setPageStep(2)
        self.apbt_slider_value.setProperty("value", 10)
        self.apbt_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.apbt_slider_value.setObjectName("apbt_slider_value")
        self.gridLayout_3.addWidget(self.apbt_slider_value, 7, 2, 1, 1)
        spacerItem19 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem19, 1, 5, 1, 1)
        self.apbt_label_parameter = QtWidgets.QLabel(self.tab_alignment)
        self.apbt_label_parameter.setObjectName("apbt_label_parameter")
        self.gridLayout_3.addWidget(self.apbt_label_parameter, 7, 0, 1, 1)
        spacerItem20 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem20, 2, 0, 1, 1)
        spacerItem21 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem21, 4, 0, 1, 1)
        self.apbt_label_display = QtWidgets.QLabel(self.tab_alignment)
        self.apbt_label_display.setObjectName("apbt_label_display")
        self.gridLayout_3.addWidget(self.apbt_label_display, 7, 4, 1, 1)
        self.aphbw_label_parameter = QtWidgets.QLabel(self.tab_alignment)
        self.aphbw_label_parameter.setObjectName("aphbw_label_parameter")
        self.gridLayout_3.addWidget(self.aphbw_label_parameter, 1, 0, 1, 1)
        spacerItem22 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem22, 6, 0, 1, 1)
        spacerItem23 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem23, 0, 0, 1, 1)
        self.gridLayout_3.setColumnStretch(0, 3)
        self.gridLayout_3.setColumnStretch(1, 1)
        self.gridLayout_3.setColumnStretch(2, 4)
        self.gridLayout_3.setColumnStretch(3, 1)
        self.gridLayout_3.setColumnStretch(4, 1)
        self.gridLayout_3.setColumnStretch(5, 7)
        self.gridLayout_3.setRowStretch(0, 1)
        self.gridLayout_3.setRowStretch(1, 3)
        self.gridLayout_3.setRowStretch(2, 1)
        self.gridLayout_3.setRowStretch(3, 3)
        self.gridLayout_3.setRowStretch(4, 1)
        self.gridLayout_3.setRowStretch(5, 3)
        self.gridLayout_3.setRowStretch(6, 1)
        self.gridLayout_3.setRowStretch(7, 3)
        self.gridLayout_3.setRowStretch(8, 12)
        self.tabWidget.addTab(self.tab_alignment, "")
        self.tab_stacking = QtWidgets.QWidget()
        self.tab_stacking.setObjectName("tab_stacking")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab_stacking)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.fnt_label_parameter = QtWidgets.QLabel(self.tab_stacking)
        self.fnt_label_parameter.setObjectName("fnt_label_parameter")
        self.gridLayout_4.addWidget(self.fnt_label_parameter, 4, 0, 1, 1)
        self.fnt_label_display = QtWidgets.QLabel(self.tab_stacking)
        self.fnt_label_display.setObjectName("fnt_label_display")
        self.gridLayout_4.addWidget(self.fnt_label_display, 4, 4, 1, 1)
        spacerItem24 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem24, 2, 0, 1, 1)
        self.fn_checkBox = QtWidgets.QCheckBox(self.tab_stacking)
        self.fn_checkBox.setObjectName("fn_checkBox")
        self.gridLayout_4.addWidget(self.fn_checkBox, 3, 0, 1, 3)
        spacerItem25 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem25, 0, 0, 1, 1)
        self.fnt_slider_value = QtWidgets.QSlider(self.tab_stacking)
        self.fnt_slider_value.setMaximum(40)
        self.fnt_slider_value.setPageStep(2)
        self.fnt_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.fnt_slider_value.setObjectName("fnt_slider_value")
        self.gridLayout_4.addWidget(self.fnt_slider_value, 4, 2, 1, 1)
        self.sfdfs_label_parameter = QtWidgets.QLabel(self.tab_stacking)
        self.sfdfs_label_parameter.setObjectName("sfdfs_label_parameter")
        self.gridLayout_4.addWidget(self.sfdfs_label_parameter, 6, 0, 1, 1)
        spacerItem26 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem26, 5, 0, 1, 1)
        spacerItem27 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem27, 7, 0, 1, 1)
        self.sfdfs_comboBox = QtWidgets.QComboBox(self.tab_stacking)
        self.sfdfs_comboBox.setObjectName("sfdfs_comboBox")
        self.gridLayout_4.addWidget(self.sfdfs_comboBox, 6, 2, 1, 1)
        spacerItem28 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem28, 4, 1, 1, 1)
        spacerItem29 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem29, 4, 3, 1, 1)
        spacerItem30 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem30, 4, 5, 1, 1)
        self.apfp_comboBox = QtWidgets.QComboBox(self.tab_stacking)
        self.apfp_comboBox.setObjectName("apfp_comboBox")
        self.gridLayout_4.addWidget(self.apfp_comboBox, 1, 0, 1, 3)
        self.apfp_spinBox = QtWidgets.QSpinBox(self.tab_stacking)
        self.apfp_spinBox.setMinimum(1)
        self.apfp_spinBox.setObjectName("apfp_spinBox")
        self.gridLayout_4.addWidget(self.apfp_spinBox, 1, 3, 1, 2)
        self.gridLayout_4.setColumnStretch(0, 2)
        self.gridLayout_4.setColumnStretch(1, 1)
        self.gridLayout_4.setColumnStretch(2, 3)
        self.gridLayout_4.setColumnStretch(3, 1)
        self.gridLayout_4.setColumnStretch(4, 2)
        self.gridLayout_4.setColumnStretch(5, 7)
        self.gridLayout_4.setRowStretch(0, 1)
        self.gridLayout_4.setRowStretch(1, 3)
        self.gridLayout_4.setRowStretch(2, 1)
        self.gridLayout_4.setRowStretch(3, 3)
        self.gridLayout_4.setRowStretch(4, 3)
        self.gridLayout_4.setRowStretch(5, 1)
        self.gridLayout_4.setRowStretch(6, 3)
        self.gridLayout_4.setRowStretch(7, 7)
        self.tabWidget.addTab(self.tab_stacking, "")
        self.GridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        self.GridLayout.setColumnStretch(0, 5)

        self.retranslateUi(ConfigurationDialog)
        self.tabWidget.setCurrentIndex(0)
        self.afm_comboBox.setCurrentIndex(-1)
        self.apsw_slider_value.valueChanged['int'].connect(self.apsw_label_display.setNum)
        self.aphbw_slider_value.valueChanged['int'].connect(self.aphbw_label_display.setNum)
        self.apbt_slider_value.valueChanged['int'].connect(self.apbt_label_display.setNum)
        self.afsw_slider_value.valueChanged['int'].connect(self.afsw_label_display.setNum)
        self.afrsf_slider_value.valueChanged['int'].connect(self.afrsf_label_display.setNum)
        self.afafp_slider_value.valueChanged['int'].connect(self.afafp_label_display.setNum)
        QtCore.QMetaObject.connectSlotsByName(ConfigurationDialog)

    def retranslateUi(self, ConfigurationDialog):
        _translate = QtCore.QCoreApplication.translate
        ConfigurationDialog.setWindowTitle(_translate("ConfigurationDialog", "Parameter Configuration"))
        self.restore_standard_values.setToolTip(_translate("ConfigurationDialog", "Reset parameters to original values. In most cases they should give satisfactory results."))
        self.restore_standard_values.setText(_translate("ConfigurationDialog", "Restore standard values"))
        self.nfs_checkBox.setText(_translate("ConfigurationDialog", "Number of frames to be stacked"))
        self.mr_checkBox.setText(_translate("ConfigurationDialog", "Maximum RAM (GBytes)"))
        self.spp_checkBox.setToolTip(_translate("ConfigurationDialog", "Postprocess the stacked image immediately after stacking,\n"
"rather than in a separate job"))
        self.spp_checkBox.setText(_translate("ConfigurationDialog", "Stacking plus postprocessing"))
        self.pfs_checkBox.setText(_translate("ConfigurationDialog", "Percent of frames to be stacked"))
        self.nap_checkBox.setText(_translate("ConfigurationDialog", "Number of alignment points"))
        self.gpspwr_checkBox.setToolTip(_translate("ConfigurationDialog", "Check if for each job the corresponding protocol section should be stored along with the stacked image."))
        self.gpspwr_checkBox.setText(_translate("ConfigurationDialog", "Store protocol with results"))
        self.ipfn_checkBox.setToolTip(_translate("ConfigurationDialog", "The user can choose which parameter values are encoded in the name of the stacking output file name."))
        self.ipfn_checkBox.setText(_translate("ConfigurationDialog", "Include parameters in output file name"))
        self.apbs_checkBox.setText(_translate("ConfigurationDialog", "Alignment point box size (pixels)"))
        self.gpbl_label_parameter.setToolTip(_translate("ConfigurationDialog", "Level of data buffering, 0 for no buffering, 4 for buffering all frames and intermediate results."))
        self.gpbl_label_parameter.setText(_translate("ConfigurationDialog", "Data buffering level"))
        self.gppl_label_parameter.setToolTip(_translate("ConfigurationDialog", "Level of detail of protocol: 0 for nothing, 1 for major steps only, 2 for detailed info."))
        self.gppl_label_parameter.setText(_translate("ConfigurationDialog", "Protocol detail level"))
        self.gpif_label_parameter.setToolTip(_translate("ConfigurationDialog", "Stacked and postprocessed images can be written either in \"png\", \"tiff\" or \"fits\" format."))
        self.gpif_label_parameter.setText(_translate("ConfigurationDialog", "Write images as"))
        self.gpwptf_checkBox.setToolTip(_translate("ConfigurationDialog", "Append a protocol of this program execution to the text file \"PlanetarySystemStacker.log\" in the user\'s home directory."))
        self.gpwptf_checkBox.setText(_translate("ConfigurationDialog", "Write protocol to file"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_workflow), _translate("ConfigurationDialog", "Workflow Parameters"))
        self.efs_checkBox.setToolTip(_translate("ConfigurationDialog", "Include a dialog where the user can select frames which should not be used in the stacking workflow."))
        self.efs_checkBox.setText(_translate("ConfigurationDialog", "Dialog to exclude frames from stacking"))
        self.afa_checkBox.setToolTip(_translate("ConfigurationDialog", "Only for \"Surface\" mode: Identify frame stabilization patch automatically."))
        self.afa_checkBox.setText(_translate("ConfigurationDialog", "Automatic frame stabilization"))
        self.afrsf_label_parameter.setToolTip(_translate("ConfigurationDialog", "Only for \"Surface\" mode: Size of the stabilization patch relative to the frame size."))
        self.afrsf_label_parameter.setText(_translate("ConfigurationDialog", "Stabilization patch size (% of frame size)"))
        self.afafp_label_parameter.setToolTip(_translate("ConfigurationDialog", "The reference frame for the multipoint alignment is computed as the average of the best frames. This value specifies the number of frames to be used."))
        self.afafp_label_parameter.setText(_translate("ConfigurationDialog", "Percentage of best frames for\n"
"reference frame computation"))
        self.fgw_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.afrsf_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.afafp_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.afsw_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.fgw_label_parameter.setToolTip(_translate("ConfigurationDialog", "In the presence of noise in the image, shift detection between frames can be misled by spurious local minima.\n"
"Gaussian blur can help finding the global optimum. Try higher values for noisy images."))
        self.fgw_label_parameter.setText(_translate("ConfigurationDialog", "Noise level (add Gaussian blur)"))
        self.afsw_label_parameter.setToolTip(_translate("ConfigurationDialog", "Only for \"Surface\" mode: Maximum allowed shift of consecutive frames per coordinate direction in pixels."))
        self.afsw_label_parameter.setText(_translate("ConfigurationDialog", "Stabilization search width (pixels)"))
        self.fdb_label_parameter.setToolTip(_translate("ConfigurationDialog", "Default debayering pattern used if no other pattern is specified for the current job. This pattern is used in creating master frames as well."))
        self.fdb_label_parameter.setText(_translate("ConfigurationDialog", "Debayering default"))
        self.afm_label_parameter.setToolTip(_translate("ConfigurationDialog", "All frames have to be globally aligned with each other before stacking. Two modes are supported: \"Surface\" and \"Planetary\".\n"
"In \"Surface\" mode (moon, sun) a patch with sufficient local contrast is used for image registration.\n"
"In \"Planetary\" mode, the \"centers of gravity\" of all frames are registered with each other."))
        self.afm_label_parameter.setText(_translate("ConfigurationDialog", "Frame stabilization  mode"))
        self.fco_checkBox.setToolTip(_translate("ConfigurationDialog", "The time span from which frames are taken for computing the reference frame is restricted to avoid blur."))
        self.fco_checkBox.setText(_translate("ConfigurationDialog", "Object is changing fast (e.g. Jupiter, Sun)"))
        self.fdbm_label_parameter.setText(_translate("ConfigurationDialog", "Debayering method"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_frames), _translate("ConfigurationDialog", "Frame-related Parameters"))
        self.apst_label_parameter.setToolTip(_translate("ConfigurationDialog", "In the automatic construction of alignment point grids, points are discarded if there is too little local structure.\n"
"Higher values result in more points to be discarded. If the value is too low, unreliable alignment points are included."))
        self.apst_label_parameter.setText(_translate("ConfigurationDialog", "Minimum structure"))
        self.apsw_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.apsw_label_parameter.setToolTip(_translate("ConfigurationDialog", "Maximum allowed local warp per coordinate direction."))
        self.apsw_label_parameter.setText(_translate("ConfigurationDialog", "Max. alignment search width (pixels)"))
        self.apst_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.aphbw_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.apbt_label_parameter.setToolTip(_translate("ConfigurationDialog", "In the automatic construction of alignment point grids, points are discarded if no pixels in the surrounding box are bright enough.\n"
"Higher values result in more points to be discarded. If the value is too low, unreliable alignment points are included."))
        self.apbt_label_parameter.setText(_translate("ConfigurationDialog", "Minimum brightness"))
        self.apbt_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.aphbw_label_parameter.setToolTip(_translate("ConfigurationDialog", "Size of the quadratic box around an alignment point used to measure the local shift.\n"
"Smaller values result in better resolution of locality, but shift detection can fail if local contrast is low."))
        self.aphbw_label_parameter.setText(_translate("ConfigurationDialog", "Alignment box width (pixels)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_alignment), _translate("ConfigurationDialog", "Multipoint Alignment Parameters"))
        self.fnt_label_parameter.setToolTip(_translate("ConfigurationDialog", "Pixels with brightness below this value will not be included in normalization.\n"
"This reduces the effect of background noise in the dark space around the object."))
        self.fnt_label_parameter.setText(_translate("ConfigurationDialog", "Normalization black cut-off"))
        self.fnt_label_display.setText(_translate("ConfigurationDialog", "TextLabel"))
        self.fn_checkBox.setToolTip(_translate("ConfigurationDialog", "Normalize the brightness of all frames. This will improve the ranking of frames and alignment patches, and reduce frame blending artifacts in stacking."))
        self.fn_checkBox.setText(_translate("ConfigurationDialog", "Normalize frame brightness"))
        self.sfdfs_label_parameter.setToolTip(_translate("ConfigurationDialog", "During stacking, the number of pixels in each coordinate direction is multiplied with this factor."))
        self.sfdfs_label_parameter.setText(_translate("ConfigurationDialog", "Use drizzling factor in stacking"))
        self.apfp_comboBox.setToolTip(_translate("ConfigurationDialog", "At each alignment point the same number of frames are stacked.\n"
"This value can either be specified explicitly or by the percentage\n"
"of the total number of frames."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_stacking), _translate("ConfigurationDialog", "Stacking Parameters"))



from cv2 import cvtColor, imread, COLOR_BGR2RGB, imshow, destroyAllWindows, \
    waitKey, COLOR_RGB2BGR
from os.path import splitext

from miscellaneous import Miscellaneous
from frames import Frames
from exceptions import Error

def test_detailed(input_file_name, shift_red, shift_blue, interpolation_factor, reduction_factor):
    # Change colors to standard RGB
    input_image = cvtColor(imread(input_file_name, -1), COLOR_BGR2RGB)
    image_original = Miscellaneous.shift_colors(input_image, (0, 0), (0, 0),
                                                interpolate_input=interpolation_factor,
                                                reduce_output=reduction_factor)
    imshow('Original image', cvtColor(image_original, COLOR_RGB2BGR))
    waitKey()
    destroyAllWindows()

    print("Input shifts:")
    print("Red channel shift: (" + str(round(shift_red[0], 1)) + ", " +
          str(round(shift_red[1], 1)) + ")")
    print("Blue channel shift: (" + str(round(shift_blue[0], 1)) + ", " +
          str(round(shift_blue[1], 1)) + ")")

    image_shifted = Miscellaneous.shift_colors(input_image, shift_red, shift_blue,
                                               interpolate_input=interpolation_factor)

    imshow('RGB shifted image', cvtColor(image_shifted, COLOR_RGB2BGR))
    waitKey()
    destroyAllWindows()

    max_shift = 5 * interpolation_factor
    blurr_strength = 7
    reference_channel = 1

    print("\nMeasured shifts:")
    channel_red = 0
    try:
        shift_red = Miscellaneous.measure_rgb_shift(image_shifted, channel_red, reference_channel,
                                                    max_shift, blur_strength=blurr_strength)
        print("Red channel shift: (" + str(round(shift_red[0] / interpolation_factor, 1)) + ", " +
              str(round(shift_red[1] / interpolation_factor, 1)) + ")")
    except Error as e:
        print("Error in measuring red shift: " + e.message)
    channel_blue = 2
    try:
        shift_blue = Miscellaneous.measure_rgb_shift(image_shifted, channel_blue, reference_channel,
                                                     max_shift, blur_strength=blurr_strength)
        print("Blue channel shift: (" + str(round(shift_blue[0] / interpolation_factor, 1)) + ", " +
              str(round(shift_blue[1] / interpolation_factor, 1)) + ")")
    except Error as e:
        print("Error in measuring blue shift: " + e.message)

    image_corrected = Miscellaneous.shift_colors(image_shifted, (-shift_red[0], -shift_red[1]),
                                                 (-shift_blue[0], -shift_blue[1]))

    imshow('RGB shifted image', cvtColor(image_corrected, COLOR_RGB2BGR))
    waitKey()
    destroyAllWindows()

def create_shifted_image(input_file_name, shift_red, shift_blue, interpolation_factor):
    # Change colors to standard RGB
    input_image = cvtColor(imread(input_file_name, -1), COLOR_BGR2RGB)
    shifted_image = Miscellaneous.shift_colors(input_image, shift_red, shift_blue,
                               interpolate_input=interpolation_factor,
                               reduce_output=interpolation_factor)

    output_file_name= splitext(input_file_name)[0] + '_rgb-shifted.png'
    Frames.save_image(output_file_name, shifted_image,
                      color=(len(shifted_image.shape) == 3), avoid_overwriting=False,
                      header="PlanetarySystemStacker")

def test_auto_rgbg_align(input_file_name, interpolation_factor, blur_strength):
    # Change colors to standard RGB
    shifted_image = cvtColor(imread(input_file_name, -1), COLOR_BGR2RGB)

    imshow('RGB-aligned image', cvtColor(shifted_image, COLOR_RGB2BGR))
    waitKey()
    destroyAllWindows()

    max_shift = 5
    corrected_image, shift_red, shift_blue = Miscellaneous.auto_rgb_align(shifted_image, max_shift,
        interpolation_factor=interpolation_factor, blur_strenght=blur_strength)

    output_file_name = splitext(input_file_name)[0] + '_corrected.png'
    Frames.save_image(output_file_name, corrected_image,
                      color=(len(corrected_image.shape) == 3), avoid_overwriting=False,
                      header="PlanetarySystemStacker")

    print("Corrections applied:")
    print("Red channel shift: (" + str(round(shift_red[0], 1)) + ", " +
          str(round(shift_red[1], 1)) + ")")
    print("Blue channel shift: (" + str(round(shift_blue[0], 1)) + ", " +
          str(round(shift_blue[1], 1)) + ")")

    imshow('RGB-aligned image', cvtColor(corrected_image, COLOR_RGB2BGR))
    waitKey()
    destroyAllWindows()

input_file_name = "D:\SW-Development\Python\PlanetarySystemStacker\Examples\Jupiter_Richard\\2020-07-29-2145_3-L-Jupiter_ALTAIRGP224C_pss_p70_b48.png"
input_file_name_shifted = "D:\SW-Development\Python\PlanetarySystemStacker\Examples\Jupiter_Richard\\2020-07-29-2145_3-L-Jupiter_ALTAIRGP224C_pss_p70_b48_rgb-shifted.png"

# input_file_name = "D:\SW-Development\Python\PlanetarySystemStacker\Examples\Jupiter_Richard\\2020-07-29-2145_3-L-Jupiter_ALTAIRGP224C_pss_p70_b48.png"
# input_file_name_shifted = "D:\SW-Development\Python\PlanetarySystemStacker\Examples\Jupiter_Richard\\2020-07-29-2145_3-L-Jupiter_ALTAIRGP224C_pss_p70_b48_rgb-shifted.png"

interpolation_factor = 4
reduction_factor = 1
blur_strength = 1
shift_red = (1.3, -3.2)
shift_blue = (-1.8, 1.1)

# test_detailed(input_file_name, shift_red, shift_blue, interpolation_factor, reduction_factor)
create_shifted_image(input_file_name, shift_red, shift_blue, interpolation_factor)
test_auto_rgbg_align(input_file_name_shifted, interpolation_factor, blur_strength)


# -*- coding: utf-8; -*-
"""
Copyright (c) 2021 Rolf Hempel, rolf6419@gmx.de

This file is part of the PlanetarySystemStacker tool (PSS).
https://github.com/Rolf-Hempel/PlanetarySystemStacker

PSS is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PSS.  If not, see <http://www.gnu.org/licenses/>.

"""

from sys import argv

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt

from configuration import Configuration
from quickstart_gui import Ui_Form


class DisplayQuickstart(QtWidgets.QWidget, Ui_Form):
    def __init__(self, parent_gui, configuration):
        """
        Initialization of the widget. The quickstart guide is displayed in the viewer window.
        """

        super(DisplayQuickstart, self).__init__()
        self.setupUi(self)

        self.parent_gui = parent_gui
        self.configuration = configuration

        # Get the quickstart guide text from a markdown document. The original text is in file
        # "quickstart.md".
        self.label.setTextFormat(Qt.MarkdownText)
        markdown_text = """
### Quickstart Guide

#### Configuration
Select 'Edit / Edit configuration' to check if the configuration parameters are set properly. 
The dialog starts with the 'Frame-related Parameters' where the most important choice is the
stabilization mode:
* Surface (for extended objects which do not fit into the FoV)
* Planet (if there is space around the object on all sides)

If the view is very unsteady (e.g. caused by wind), increase the 'Stabilization search width'.
For noisier planetary images, try 'Noise levels' up to the maximum. Check the box
'Dialog to exclude frames from stacking' if some frames may be corrupted by artifacts.
Have a look at the other parameter sections as well. For example, for strongly oversampled
planetary images consider increasing the 'Max alignment search width' (Section 'Multipoint Alignment
Parameters').

#### Select jobs
Next, press 'File / Open' to specify the jobs for this session. Video files / image folders are
stacked, single images are postprocessed. Either use the PSS file chooser, or cancel the chooser and
'drag and drop' objects from a file explorer.

#### Job execution (interactive / batch)
Start the processing with 'Start / Cont.'. Jobs are executed in consecutive order, either
interactively (default, with GUI), or automatically (batch). If jobs are similar, use interactive
mode for the first job to make processing choices. Then check the box 'Automatic' to have PSS
process all the other jobs automatically with the same parameters. You can change back to
interactive mode at any time by unchecking the 'Automatic' box.

The most important dialog is the one where the alignment points are set. It is critical
to experiment with different 'Alignment box widths' (upper right slider). Start with a high
value (e.g. 100), and reduce it until the final image quality does not improve any further.
Too low values may result in stacking artifacts. As an alternative to the automatic AP
grid generation, you can set APs manually by left-clicking into the preview image.

Another important paramter which needs to be adapted to the individual case is the number /
percentage of frames used for stacking. Try different values to find the optimal choice.

#### Postprocessing
Postprocessing can follow stacking immediately (if the workflow parameter 'Stacking plus
postprocessing' is checked), or be executed separately (if the job input is a single image file).
To process several images with the same parameters, adjust the parameters for the first image in
interactive mode, and then check 'Automatic' to repeat the same for all other images in batch mode. 
        """

        self.label.setText(markdown_text)

        self.dont_show_checkBox.setChecked(not self.configuration.global_parameters_display_quickstart)
        self.dont_show_checkBox.stateChanged.connect(self.checkbox_changed)

    def checkbox_changed(self):
        """
        This method is connected with the checkbox in the quickstart guide and with the menu
        entry where the quickstart guide can be re-activated.

        :return: -
        """
        self.configuration.global_parameters_display_quickstart = \
            not self.configuration.global_parameters_display_quickstart
        if self.parent_gui is not None:
            self.parent_gui.ui.actionShow_Quickstart.setChecked(
                self.configuration.global_parameters_display_quickstart)


if __name__ == '__main__':
    # Get configuration parameters.
    configuration = Configuration()
    configuration.initialize_configuration()

    app = QtWidgets.QApplication(argv)
    window = DisplayQuickstart(None, configuration)
    window.show()
    app.exec_()



from frames import Frames
import pyfits
from numpy import moveaxis

import cv2
from cv2 import cvtColor, COLOR_RGB2BGR, COLOR_BGR2RGB

def astropy_fits(filename_in, filename_out):
    frame = cvtColor(Frames.read_image(filename_in), COLOR_RGB2BGR)

    cv2.imshow('Example - Show image in window', frame)

    cv2.waitKey(0)
    cv2.destroyAllWindows()

    Frames.save_image(filename_out, cvtColor(frame, COLOR_BGR2RGB), color=True)

def pyfits_fits(filename_in, filename_out):
    frame = cvtColor(pyfits.getdata(filename_in), COLOR_RGB2BGR)

    cv2.imshow('Example - Show image in window', frame)

    cv2.waitKey(0)
    cv2.destroyAllWindows()

    if len(frame.shape) == 3:
        frame = moveaxis(frame, -1, 0)
    hdu = pyfits.PrimaryHDU(frame)
    header = "PlanetarySystemStacker"
    hdu.header['CREATOR'] = header
    hdu.writeto(filename_out, overwrite=True)

# filename = "D:\SW-Development\Python\PlanetarySystemStacker\Examples\Jupiter_Richard\\2020-07-29-2145_3-L-Jupiter_ALTAIRGP224C_pss_p70_b48_gpp.fits"
filename = 'frame.fits'
# astropy_fits(filename, "frame_out.fits")
pyfits_fits(filename, "frame_out.fits")


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

release_tag = "0.9.8.2"

setuptools.setup(
    name="planetary-system-stacker",
    version=release_tag,
    author="Rolf Hempel",
    author_email="rolf6419@gmx.de",
    description="PlanetarySystemStacker",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Rolf-Hempel/PlanetarySystemStacker",
    packages=setuptools.find_packages(),
    ext_modules=None,
    install_requires=[
        'numpy < 1.23.0',
        'matplotlib',
        'psutil',
        'PyQt5 >= 5.15',
        'scipy',
        'astropy',
        'scikit-image',
        'opencv-python-headless'
    ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        # "Operating System :: OS Independent"
        "Operating System :: POSIX :: Linux"
    ],
    python_requires='>=3.5',
    entry_points={
        "console_scripts": [
        "PlanetarySystemStacker=planetary_system_stacker.planetary_system_stacker:main",
        ]
    },
)



# -*- coding: utf-8; -*-
"""
Copyright (c) 2018 Rolf Hempel, rolf6419@gmx.de

This file is part of the PlanetarySystemStacker tool (PSS).
https://github.com/Rolf-Hempel/PlanetarySystemStacker

PSS is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PSS.  If not, see <http://www.gnu.org/licenses/>.

"""

class Error(Exception):
    """Base class for exceptions in this module.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class TypeError(Error):
    """Exception raised for type errors during input.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        super(TypeError, self).__init__(message)

class ShapeError(Error):
    """Exception raised if image shapes do not match during input.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        super(ShapeError, self).__init__(message)

class NotSupportedError(Error):
    """Exception raised if a method is called which is not implemented yet.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        super(NotSupportedError, self).__init__(message)

class ArgumentError(Error):
    """Exception raised if a method is called which an invalid argument.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        super(ArgumentError, self).__init__(message)

class WrongOrderingError(Error):
    """Exception raised if a method is called before required input is available.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        super(WrongOrderingError, self).__init__(message)

class InternalError(Error):
    """Exception raised if an internal error occurred which should never happen.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        super(InternalError, self).__init__(message)

class DivideByZeroError(Error):
    """Exception raised if an attempt is made to divide by zero.

        Attributes:
            message -- explanation of the error
        """

    def __init__(self, message):
        super(DivideByZeroError, self).__init__(message)

class IncompatibleVersionsError(Error):
    """Exception raised if data and program versions do not match.

        Attributes:
            message -- explanation of the error
        """

    def __init__(self, message):
        super(IncompatibleVersionsError, self).__init__(message)



# -*- coding: utf-8 -*-
"""
@author: Michal Powalko m.powalko@gmail.com

Version 1.0 [2019-08-01]
------------------------
    INF: Inititial release

"""

import cv2
import time
import numpy as np
from scipy import ndimage
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

# https://docs.opencv.org/3.1.0/dd/d49/tutorial_py_contour_features.html
# https://www.learnopencv.com/find-center-of-blob-centroid-using-opencv-cpp-python/
# https://docs.opencv.org/3.4.0/d7/d4d/tutorial_py_thresholding.html
# https://en.wikipedia.org/wiki/Image_moment#Examples

LOOPS = int(1)

file_path = r'D:\SW-Development\Python\PlanetarySystemStacker\Examples\Jupiter\2019-05-26-0115_4-L-Jupiter_ZWO ASI290MM Mini_pipp.avi'
cap = cv2.VideoCapture(file_path)
frame_count = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))

tic = time.time()
frames = [cv2.cvtColor(cap.read()[1], cv2.COLOR_BGR2GRAY) for idx in range(frame_count)]
print('Importing the video file took:\t\t{0:.2f} [s]'.format(time.time() - tic))
cap.release()

tic = time.time()
frames = [cv2.GaussianBlur(frame, (3, 3), 0) for frame in frames]
print('GaussianBlur took:\t\t\t{0:.2f} [s]'.format(time.time() - tic))

tic = time.time()
for idx in range(LOOPS):
    cog_rolf = [ndimage.measurements.center_of_mass(np.uint8(1)*(frame >= (np.max(frame)/2).astype(frame.dtype)))
                for frame in frames]
    cog_rolf = [(int(round(aaa[0])), int(round(aaa[1]))) for aaa in cog_rolf]
print('Rolf/Jens CoG estimation took:\t\t{0:.2f} [s]'.format(time.time() - tic))

tic = time.time()
for idx in range(LOOPS):
    MM = [cv2.moments(cv2.threshold(frame, frame.max()/2, 1, cv2.THRESH_BINARY)[1])
         for frame in frames]
    cog_michal = [(round(M["m01"] / M["m00"]), round(M['m10'] / M["m00"])) for M in MM]
print('Michals CoG estimation took:\t\t{0:.2f} [s]'.format(time.time() - tic))

dev_x = [a[1] - b[1] for a,b in zip(cog_rolf, cog_michal)]
dev_y = [a[0] - b[0] for a,b in zip(cog_rolf, cog_michal)]

plt.subplot(2, 1, 1)
plt.title('Center of gravity estimation')
plt.plot(dev_x, '-b')
plt.xlabel('Frame chronological number')
plt.ylabel('Error in x axis [pixel]')
plt.grid(True)
plt.subplot(2, 1, 2)
plt.title('Center of gravity estimation')
plt.plot(dev_y, '-r')
plt.xlabel('Frame chronological number')
plt.ylabel('Error in y axis [pixel]')
plt.grid(True)
pass




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'frame_viewer_test_gui.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(270, 170)
        self.gridLayout = QtWidgets.QGridLayout(Frame)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(Frame)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.retranslateUi(Frame)
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        _translate = QtCore.QCoreApplication.translate
        Frame.setWindowTitle(_translate("Frame", "Frame"))
        self.label.setText(_translate("Frame", "textLabel"))



import argparse
import sys

print("The command was called with " + str(len(sys.argv)) + " arguments")

if len(sys.argv) <= 1:
    print ("Running PSS in interactive mode, starting GUI.")
    exit()

def noise_type(x):
    x = int(x)
    if not 0 <= x <= 11:
        raise argparse.ArgumentTypeError("Noise level must be between 0 and 11")
    return x


def stab_size_type(x):
    x = int(x)
    if not 5 <= x <= 80:
        raise argparse.ArgumentTypeError("Stabilization patch size must be between 5% and 80%")
    return x


def stab_sw_type(x):
    x = int(x)
    if not 5 <= x <= 150:
        raise argparse.ArgumentTypeError(
            "Stabilization search width must be between 5 and 150 pixels")
    return x


def rf_percent_type(x):
    x = int(x)
    if not 3 <= x <= 30:
        raise argparse.ArgumentTypeError(
            "Percentage of best frames for reference frame computation must be between 3% and 30%")
    return x


def align_box_width_type(x):
    x = int(x)
    if not 20 <= x <= 140:
        raise argparse.ArgumentTypeError(
            "Alignment point box width must be between 20 and 140 pixels")
    return x


def align_search_width_type(x):
    x = int(x)
    if not 6 <= x <= 30:
        raise argparse.ArgumentTypeError(
            "Alignment point search width must be between 6 and 30 pixels")
    return x


def align_min_struct_type(x):
    x = float(x)
    if not 0.01 <= x <= 0.30:
        raise argparse.ArgumentTypeError(
            "Alignment point minimum structure must be between 0.01 and 0.30")
    return x


def align_min_bright_type(x):
    x = int(x)
    if not 2 <= x <= 50:
        raise argparse.ArgumentTypeError(
            "Alignment point minimum brightness must be between 2 and 50")
    return x


def stack_percent_type(x):
    x = int(x)
    if not 1 <= x <= 100:
        raise argparse.ArgumentTypeError(
            "Percentage of best frames to be stacked must be between 1 and 100")
    return x

def stack_number_type(x):
    x = int(x)
    if not 1 <= x:
        raise argparse.ArgumentTypeError(
            "Number of best frames to be stacked must be greater or equal 1")
    return x

def normalize_bco_type(x):
    x = int(x)
    if not 0 <= x <= 40:
        raise argparse.ArgumentTypeError(
            "Normalization black cut-off must be between 0 and 40")
    return x


parser = argparse.ArgumentParser()
parser.add_argument("job_input", nargs='+', help="input video files or still image folders")

parser.add_argument("-p", "--protocol", action="store_true",
                    help="Store protocol with results")
parser.add_argument("--protocol_detail", type=int, choices=[0, 1, 2], default=1,
                    help="Protocol detail level")
parser.add_argument("-b", "--buffering_level", type=int, choices=[0, 1, 2, 3, 4], default=2,
                    help="Buffering level")
parser.add_argument("--out_format", choices=["png", "tiff", "fits"], default="png",
                    help="Image format for output")
parser.add_argument("--name_add_f", action="store_true",
                    help="Add number of stacked frames to output file name")
parser.add_argument("--name_add_p", action="store_true",
                    help="Add percentage of stacked frames to output file name")
parser.add_argument("--name_add_apb", action="store_true",
                    help="Add alignment point box size (pixels) to output file name")
parser.add_argument("--name_add_apn", action="store_true",
                    help="Add number of alignment points to output file name")

parser.add_argument("--debayering", choices=["Auto detect color", "Grayscale", "RGB", "RGB", "BGR",
                                             "Force Bayer RGGB", "Force Bayer GRBG",
                                             "Force Bayer GBRG", "Force Bayer BGGR"],
                    default="Auto detect color", help="Debayering option")
parser.add_argument("--noise", type=noise_type, default=7, help="Noise level (add Gaussian blur)")
parser.add_argument("-m", "--stab_mode", choices=["Surface", "Planet"], default="Surface",
                    help="Frame stabilization mode")
parser.add_argument("--stab_size", type=stab_size_type, default=33,
                    help="Stabilization patch size (% of frame)")
parser.add_argument("--stab_sw", type=stab_sw_type, default=34,
                    help="Stabilization search width (pixels)")
parser.add_argument("--rf_percent", type=rf_percent_type, default=5,
                    help="Percentage of best frames for reference frame computation")
parser.add_argument("-d", "--dark", help="Image file for dark frame correction")
parser.add_argument("-f", "--flat", help="Image file for flat frame correction")

parser.add_argument("-a", "--align_box_width", type=align_box_width_type, default=48,
                    help="Alignment point box width (pixels)")
parser.add_argument("-w", "--align_search_width", type=align_search_width_type, default=14,
                    help="Alignment point search width (pixels)")
parser.add_argument("--align_min_struct", type=align_min_struct_type, default=0.04,
                    help="Alignment point minimum structure")
parser.add_argument("--align_min_bright", type=align_min_bright_type, default=10,
                    help="Alignment point minimum brightness")

parser.add_argument("-s", "--stack_percent", type=stack_percent_type, default=10,
                    help="Percentage of best frames to be stacked")
parser.add_argument("--stack_number", type=stack_number_type,
                    help="Number of best frames to be stacked")
parser.add_argument("-n", "--normalize_bright", action="store_true",
                    help="Normalize frame brightness")
parser.add_argument("--normalize_bco", type=normalize_bco_type, default=15,
                    help="Normalization black cut-off")

args = parser.parse_args()

print(str(args.job_input))
print("Store protocol with results: " + str(args.protocol))
print("Protocol detail level: " + str(args.protocol_detail))
print("Buffering level: " + str(args.buffering_level))
print("Image format for output: " + args.out_format)
print("Add number of stacked frames to output file name: " + str(args.name_add_f))
print("Add percentage of stacked frames to output file name: " + str(args.name_add_p))
print("Add alignment point box size (pixels) to output file name: " + str(args.name_add_apb))
print("Add number of alignment points to output file name: " + str(args.name_add_apn))
print("")
print("Debayering option: " + args.debayering)
print("Noise level: " + str(args.noise))
print("Frame stabilization mode: " + args.stab_mode)
print("Stabilization patch size (% of frame): " + str(args.stab_size))
print("Stabilization search width (pixels): " + str(args.stab_sw))
print("Percentage of best frames for reference frame computation: " + str(args.rf_percent))
if args.dark:
    print("Image file for dark frame correction: " + args.dark)
if args.flat:
    print("Image file for flat frame correction: " + args.flat)
print("")
print("Alignment point box width (pixels): " + str(args.align_box_width))
print("Alignment point search width (pixels): " + str(args.align_search_width))
print("Alignment point minimum structure: " + str(args.align_min_struct))
print("Alignment point minimum brightness: " + str(args.align_min_bright))
print("")
print("Percentage of best frames to be stacked: " + str(args.stack_percent))
print("Number of best frames to be stacked: " + str(args.stack_number))
print("Normalize frame brightness: " + str(args.normalize_bright))
print("Normalization black cut-off: " + str(args.normalize_bco))


# -*- coding: utf-8; -*-
"""
Copyright (c) 2018 Rolf Hempel, rolf6419@gmx.de

This file is part of the PlanetarySystemStacker tool (PSS).
https://github.com/Rolf-Hempel/PlanetarySystemStacker

PSS is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PSS.  If not, see <http://www.gnu.org/licenses/>.

"""

from os import path, remove
from os.path import splitext
from pathlib import Path

from astropy.io import fits
from cv2 import imread, cvtColor, COLOR_RGB2GRAY, COLOR_BGR2RGB, THRESH_TOZERO, threshold, \
    GaussianBlur, Laplacian, CV_32F, COLOR_RGB2BGR, imwrite, convertScaleAbs, IMREAD_UNCHANGED, flip
from cv2 import mean as cv_mean
from numpy import uint8, uint16, clip, moveaxis

from configuration import Configuration
from exceptions import TypeError, ArgumentError, Error
from frames import ImageReader, VideoReader
from rank_frames import RankFrames
from timer import timer


class Frames(object):
    """
        This object stores the image data of all frames. Four versions of the original frames are
        used throughout the data processing workflow. They are (re-)used in the folliwing phases:
        1. Original (color) frames, type: uint8 / uint16
            - Frame stacking ("stack_frames.stack_frames")
        2. Monochrome version of 1., type: uint8 / uint16
            - Computing the average frame (only average frame subset, "align_frames.average_frame")
        3. Gaussian blur added to 2., type: type: uint16
            - Aligning all frames ("align_frames.align_frames")
            - Frame stacking ("stack_frames.stack_frames")
        4. Down-sampled Laplacian of 3., type: uint8
            - Overall image ranking ("rank_frames.frame_score")
            - Ranking frames at alignment points("alignment_points.compute_frame_qualities")

        Buffering at various levels is available. It is controlled with four flags set at object
        initialization time.

        A complete PSS execution processes all "n" frames in four complete passes. Additionally,
        in module "align_frames" there are some extra accesses:

        1. In "rank_frames.frame_score": Access to all "Laplacians of Gaussian" (frame 0 to n-1)
           In "align_frames.select_alignment_rect and .align_frames": Access to the Gaussian of the
           best frame.
        2. In "align_frames.align_frames": Access to all Gaussians (frame 0 to n-1)
           In "align_frames.average_frame": Access to the monochrome frames of the best images for
           averaging
        3. In "alignment_points.compute_frame_qualities": Access to all "Laplacians of Gaussian"
           (frame 0 to n-1)
        4. In "stack_frames.stack_frames": Access to all frames + Gaussians (frame 0 to n-1)

    """

    @staticmethod
    def set_buffering(buffering_level):
        """
        Decide on the objects to be buffered, depending on "buffering_level" configuration
        parameter.

        :param buffering_level: Buffering level parameter as set in configuration.
        :return: Tuple of four booleans:
                 buffer_original: Keep all original frames in buffer.
                 buffer_monochrome: Keep the monochrome version of  all original frames in buffer.
                 buffer_gaussian: Keep the monochrome version with Gaussian blur added of  all
                                  frames in buffer.
                 buffer_laplacian: Keep the Laplacian of Gaussian (LoG) of the monochrome version of
                                   all frames in buffer.
        """

        buffer_original = False
        buffer_monochrome = False
        buffer_gaussian = False
        buffer_laplacian = False

        if buffering_level > 0:
            buffer_laplacian = True
        if buffering_level > 1:
            buffer_gaussian = True
        if buffering_level > 2:
            buffer_original = True
        if buffering_level > 3:
            buffer_monochrome = True

        return buffer_original, buffer_monochrome, buffer_gaussian, buffer_laplacian

    def __init__(self, configuration, names, type='video', bayer_option_selected="Auto detect color",
                 calibration=None, progress_signal=None,
                 buffer_original=True, buffer_monochrome=False, buffer_gaussian=True,
                 buffer_laplacian=True):
        """
        Initialize the Frame object, and read all images. Images can be stored in a video file or
        as single images in a directory.

        :param configuration: Configuration object with parameters
        :param names: In case "video": name of the video file. In case "image": list of names for
                      all images.
        :param type: Either "video" or "image".
        :param bayer_option_selected: Bayer pattern, one out of: "Auto detect color", "Grayscale",
                              "RGB", "BGR", "Force Bayer RGGB", "Force Bayer GRBG",
                               "Force Bayer GBRG", "Force Bayer BGGR".
        :param calibration: (Optional) calibration object for darks/flats correction.
        :param progress_signal: Either None (no progress signalling), or a signal with the signature
                                (str, int) with the current activity (str) and the progress in
                                percent (int).
        :param buffer_original: If "True", read the original frame data only once, otherwise
                                read them again if required.
        :param buffer_monochrome: If "True", compute the monochrome image only once, otherwise
                                  compute it again if required. This may include re-reading the
                                  original image data.
        :param buffer_gaussian: If "True", compute the gaussian-blurred image only once, otherwise
                                compute it again if required. This may include re-reading the
                                original image data.
        :param buffer_laplacian: If "True", compute the "Laplacian of Gaussian" only once, otherwise
                                 compute it again if required. This may include re-reading the
                                 original image data.
        """

        self.configuration = configuration
        self.names = names
        self.calibration = calibration
        self.progress_signal = progress_signal
        self.type = type
        self.bayer_pattern = None
        self.bayer_option_selected = bayer_option_selected

        self.buffer_original = buffer_original
        self.buffer_monochrome = buffer_monochrome
        self.buffer_gaussian = buffer_gaussian
        self.buffer_laplacian = buffer_laplacian

        # In non-buffered mode, the index of the image just read/computed is stored for re-use.
        self.original_available = None
        self.original_available_index = -1
        self.monochrome_available = None
        self.monochrome_available_index = -1
        self.gaussian_available = None
        self.gaussian_available_index = -1
        self.laplacian_available = None
        self.laplacian_available_index = None

        # Set a flag that no monochrome image has been computed before.
        self.first_monochrome = True

        # Initialize the list of original frames.
        self.frames_original = None

        # Compute the scaling value for Laplacian computation.
        self.alpha = 1. / 256.

        # Initialize and open the reader object.
        if self.type == 'image':
            self.reader = ImageReader(self.configuration)
        elif self.type == 'video':
            self.reader = VideoReader(self.configuration)
        else:
            raise TypeError("Image type " + self.type + " not supported")

        self.number, self.color, self.dt0, self.shape, self.shift_pixels = self.reader.open(self.names,
            bayer_option_selected=self.bayer_option_selected)

        # Look up the Bayer pattern the reader has identified.
        self.bayer_pattern = self.reader.bayer_pattern

        # Set the depth value of all images to either 16 or 8 bits.
        if self.dt0 == 'uint16':
            self.depth = 16
        elif self.dt0 == 'uint8':
            self.depth = 8
        else:
            raise TypeError("Frame type " + str(self.dt0) + " not supported")

        # Check if the darks / flats of the calibration object match the current reader.
        if self.calibration:
            self.calibration_matches = self.calibration.flats_darks_match(self.color, self.shape)
            # If there are matching darks or flats, adapt their type to the current frame type.
            if self.calibration_matches:
                self.calibration.adapt_dark_frame(self.dt0, self.shift_pixels)
        else:
            self.calibration_matches = False

        # Initialize lists of monochrome frames (with and without Gaussian blur) and their
        # Laplacians.
        colors = ['red', 'green', 'blue', 'panchromatic']
        if self.configuration.frames_mono_channel in colors:
            self.color_index = colors.index(self.configuration.frames_mono_channel)
        else:
            raise ArgumentError("Invalid color selected for channel extraction")
        self.frames_monochrome = [None] * self.number
        self.frames_monochrome_blurred = [None] *self.number
        self.frames_monochrome_blurred_laplacian = [None] *self.number
        if self.configuration.frames_normalization:
            self.frames_average_brightness = [None] *self.number
        else:
            self.frames_average_brightness = None
        self.first_monochrome_index = None
        self.used_alignment_points = None

    def compute_required_buffer_size(self, buffering_level):
        """
        Compute the RAM required to store original images and their derivatives, and other objects
        which scale with the image size.

        Additional to the original images and their derivatives, the following large objects are
        allocated during the workflow:
            calibration.master_dark_frame: pixels * colors (float32)
            calibration.master_dark_frame_uint8: pixels * colors (uint8)
            calibration.master_dark_frame_uint16: pixels * colors (uint16)
            calibration.master_flat_frame: pixels * colors (float32)
            align_frames.mean_frame: image pixels (int32)
            align_frames.mean_frame_original: image pixels (int32)
            alignment_points, reference boxes: < 2 * image pixels (int32)
            alignment_points, stacking buffers: < 2 * image pixels * colors (float32)
            stack_frames.stacked_image_buffer: image pixels * colors (float32)
            stack_frames.number_single_frame_contributions: image pixels (int32)
            stack_frames.sum_single_frame_weights: image pixels (float32)
            stack_frames.mask: image pixels (float32)
            stack_frames.averaged_background: image pixels * colors (float32)
            stack_frames.stacked_image: image pixels * colors (uint16)

        :param buffering_level: Buffering level parameter.
        :return: Number of required buffer space in bytes.
        """

        # Compute the number of image pixels.
        number_pixel = self.shape[0] * self.shape[1]

        # Compute the size of a monochrome image in bytes.
        image_size_monochrome_bytes = number_pixel * self.depth / 8

        # Compute the size of an original image in bytes.
        if self.color:
            image_size_bytes = 3 * image_size_monochrome_bytes
        else:
            image_size_bytes = image_size_monochrome_bytes

        # Compute the size of the monochrome images with Gaussian blur added in bytes.
        image_size_gaussian_bytes = number_pixel * 2

        # Compute the size of a "Laplacian of Gaussian" in bytes. Remember that it is down-sampled.
        image_size_laplacian_bytes = number_pixel / \
                                     self.configuration.align_frames_sampling_stride ** 2

        # Compute the buffer space per image, based on the buffering level.
        buffer_original, buffer_monochrome, buffer_gaussian, buffer_laplacian = \
            Frames.set_buffering(buffering_level)
        buffer_per_image = 0
        if buffer_original:
            buffer_per_image += image_size_bytes
        if buffer_monochrome:
            buffer_per_image += image_size_monochrome_bytes
        if buffer_gaussian:
            buffer_per_image += image_size_gaussian_bytes
        if buffer_laplacian:
            buffer_per_image += image_size_laplacian_bytes

        # Multiply with the total number of frames.
        buffer_for_all_images = buffer_per_image * self.number

        # Compute the size of additional workspace objects allocated during the workflow.
        buffer_additional_workspace = number_pixel * 57
        if self.color:
            buffer_additional_workspace += number_pixel * 58

        # Return the total buffer space required.
        return (buffer_for_all_images + buffer_additional_workspace) / 1e9

    def frames(self, index):
        """
        Read or look up the original frame object with a given index.

        :param index: Frame index
        :return: Frame with index "index".
        """

        if not 0 <= index < self.number:
            raise ArgumentError("Frame index " + str(index) + " is out of bounds")
        # print ("Accessing frame " + str(index))

        # If the original frames are to be buffered, read them in one go at the first call to this
        # method. In this case, a progress bar is displayed in the main GUI.
        if self.frames_original is None:
            if self.buffer_original:
                self.frames_original = []
                self.signal_step_size = max(int(self.number / 10), 1)
                for frame_index in range(self.number):
                    # After every "signal_step_size"th frame, send a progress signal to the main GUI.
                    if self.progress_signal is not None and frame_index % self.signal_step_size == 1:
                        self.progress_signal.emit("Read all frames",
                                                  int(round(10 * frame_index / self.number) * 10))
                    # Read the next frame. If dark/flat correction is active, do the corrections.
                    if self.calibration_matches:
                        self.frames_original.append(self.calibration.correct(
                            self.reader.read_frame(frame_index)))
                    else:
                        self.frames_original.append(self.reader.read_frame(frame_index))

                self.reader.close()

            # If original frames are not buffered, initialize an empty frame list, so frames can be
            # read later in non-consecutive order.
            else:
                self.frames_original = [None] *self.number

        # The original frames are buffered. Just return the frame.
        if self.buffer_original:
            return self.frames_original[index]

        # This frame has been cached. Just return it.
        if self.original_available_index == index:
            return self.original_available

        # The frame has not been stored for re-use, read it. If dark/flat correction is active, do
        # the corrections.
        else:
            if self.calibration_matches:
                frame = self.calibration.correct(self.reader.read_frame(index))
            else:
                frame = self.reader.read_frame(index)

            # Cache the frame just read.
            self.original_available = frame
            self.original_available_index = index

            return frame

    def frames_mono(self, index):
        """
        Look up or compute the monochrome version of the frame object with a given index.

        :param index: Frame index
        :return: Monochrome frame with index "index".
        """

        if not 0 <= index < self.number:
            raise ArgumentError("Frame index " + str(index) + " is out of bounds")

        # print("Accessing frame monochrome " + str(index))
        # The monochrome frames are buffered, and this frame has been stored before. Just return
        # the frame.
        if self.frames_monochrome[index] is not None:
            return self.frames_monochrome[index]

        # If the monochrome frame is cached, just return it.
        if self.monochrome_available_index == index:
            return self.monochrome_available

        # The frame has not been stored for re-use, compute it.
        else:

            # Get the original frame. If it is not cached, this involves I/O.
            frame_original = self.frames(index)

            # If frames are in color mode produce a B/W version.
            if self.color:
                if self.color_index == 3:
                    frame_mono = cvtColor(frame_original, COLOR_RGB2GRAY)
                else:
                    frame_mono = frame_original[:, :, self.color_index]
            # Frames are in B/W mode already
            else:
                frame_mono = frame_original

            # Normalize the overall frame brightness. The first monochrome frame for which this
            # method is invoked is taken as the reference. The average brightness of all other
            # monochrome frames is adjusted to match the brightness of the referenence.
            if self.configuration.frames_normalization:
                frame_type = frame_mono.dtype
                if self.first_monochrome:
                    if frame_type == uint8:
                        self.normalization_lower_threshold = \
                            self.configuration.frames_normalization_threshold
                        self.normalization_upper_threshold = 255
                    else:
                        self.normalization_lower_threshold = \
                            self.configuration.frames_normalization_threshold * 256
                        self.normalization_upper_threshold = 255

                    self.frames_average_brightness[index] = cv_mean(
                        threshold(frame_mono, self.normalization_lower_threshold,
                                  self.normalization_upper_threshold,
                                  THRESH_TOZERO)[1])[0]
                    # self.frames_average_brightness[index] = cv_mean(frame_mono)[0]
                    # Keep the index of the first monochrome frame as the reference index.
                    self.first_monochrome_index = index
                    self.first_monochrome = False
                # Not the first monochrome frame. Adjust brightness to match the reference.
                else:
                    self.frames_average_brightness[index] = cv_mean(
                        threshold(frame_mono, self.normalization_lower_threshold,
                                  self.normalization_upper_threshold,
                                  THRESH_TOZERO)[1])[0]
                    # self.frames_average_brightness[index] = cv_mean(frame_mono)[0]
                    frame_mono = frame_mono * (self.frames_average_brightness[self.first_monochrome_index] /
                                               self.frames_average_brightness[index])
                    # Clip the pixel values to the range allowed.
                    if frame_type == uint8:
                        clip(frame_mono, 0, 255, out=frame_mono)
                    else:
                        clip(frame_mono, 0, 65535, out=frame_mono)
                    frame_mono = frame_mono.astype(frame_type)

            # If the monochrome frames are buffered, store it at the current index.
            if self.buffer_monochrome:
                self.frames_monochrome[index] = frame_mono

            # If frames are not buffered, cache the current frame.
            else:
                self.monochrome_available_index = index
                self.monochrome_available = frame_mono

            return frame_mono

    def frames_mono_blurred(self, index):
        """
        Look up a Gaussian-blurred frame object with a given index.

        :param index: Frame index
        :return: Gaussian-blurred frame with index "index".
        """

        if not 0 <= index < self.number:
            raise ArgumentError("Frame index " + str(index) + " is out of bounds")

        # print("Accessing frame with Gaussian blur " + str(index))
        # The blurred frames are buffered, and this frame has been stored before. Just return
        # the frame.
        if self.frames_monochrome_blurred[index] is not None:
            return self.frames_monochrome_blurred[index]

        # If the blurred frame is cached, just return it.
        if self.gaussian_available_index == index:
            return self.gaussian_available

        # The frame has not been stored for re-use, compute it.
        else:

            # Get the monochrome frame. If it is not cached, this involves I/O.
            frame_mono = self.frames_mono(index)

            # If the mono image is 8bit, interpolate it to 16bit.
            if frame_mono.dtype == uint8:
                frame_mono = frame_mono.astype(uint16) * 256

            # Compute a version of the frame with Gaussian blur added.
            frame_monochrome_blurred = GaussianBlur(frame_mono,
                                                    (self.configuration.frames_gauss_width,
                                                     self.configuration.frames_gauss_width), 0)

            # If the blurred frames are buffered, store the current frame at the current index.
            if self.buffer_gaussian:
                self.frames_monochrome_blurred[index] = frame_monochrome_blurred

            # If frames are not buffered, cache the current frame.
            else:
                self.gaussian_available_index = index
                self.gaussian_available = frame_monochrome_blurred

            return frame_monochrome_blurred

    def frames_mono_blurred_laplacian(self, index):
        """
        Look up a Laplacian-of-Gaussian of a frame object with a given index.

        :param index: Frame index
        :return: LoG of a frame with index "index".
        """

        if not 0 <= index < self.number:
            raise ArgumentError("Frame index " + str(index) + " is out of bounds")

        # print("Accessing LoG number " + str(index))
        # The LoG frames are buffered, and this frame has been stored before. Just return the frame.
        if self.frames_monochrome_blurred_laplacian[index] is not None:
            return self.frames_monochrome_blurred_laplacian[index]

        # If the blurred frame is cached, just return it.
        if self.laplacian_available_index == index:
            return self.laplacian_available

        # The frame has not been stored for re-use, compute it.
        else:

            # Get the monochrome frame. If it is not cached, this involves I/O.
            frame_monochrome_blurred = self.frames_mono_blurred(index)

            # Compute a version of the frame with Gaussian blur added.
            frame_monochrome_laplacian = convertScaleAbs(Laplacian(
                frame_monochrome_blurred[::self.configuration.align_frames_sampling_stride,
                ::self.configuration.align_frames_sampling_stride], CV_32F),
                alpha=self.alpha)

            # If the blurred frames are buffered, store the current frame at the current index.
            if self.buffer_laplacian:
                self.frames_monochrome_blurred_laplacian[index] = frame_monochrome_laplacian

            # If frames are not buffered, cache the current frame.
            else:
                self.laplacian_available_index = index
                self.laplacian_available = frame_monochrome_laplacian

            return frame_monochrome_laplacian

    def reset_alignment_point_lists(self):
        """
        Every frame keeps a list with the alignment points for which this frame is among the
        sharpest ones (so it is used in stacking). Reset this list for all frames.

        :return: -
        """

        # For every frame initialize the list with used alignment points.
        self.used_alignment_points = [[] for index in range(self.number)]

    @staticmethod
    def save_image(filename, image, color=False, avoid_overwriting=True,
                   header="PlanetarySystemStacker"):
        """
        Save an image to a file. If "avoid_overwriting" is set to False, images can have either
        ".png", ".tiff" or ".fits" format.

        :param filename: Name of the file where the image is to be written
        :param image: ndarray object containing the image data
        :param color: If True, a three channel RGB image is to be saved. Otherwise, it is assumed
                      that the image is monochrome.
        :param avoid_overwriting: If True, append a string to the input name if necessary so that
                                  it does not match any existing file. If False, overwrite
                                  an existing file.
        :param header: String with information on the PSS version being used (optional).
        :return: -
        """

        if avoid_overwriting:
            # If a file or directory with the given name already exists, append the word "_file".
            if Path(filename).is_dir():
                while True:
                    filename += '_file'
                    if not Path(filename).exists():
                        break
                filename += '.jpg'
            # If it is a file, try to append "_copy.tiff" to its basename. If it still exists, repeat.
            elif Path(filename).is_file():
                suffix = Path(filename).suffix
                while True:
                    p = Path(filename)
                    filename = Path.joinpath(p.parents[0], p.stem + '_copy' + suffix)
                    if not Path(filename).exists():
                        break
            else:
                # If the file name is new and has no suffix, add ".tiff".
                suffix = Path(filename).suffix
                if not suffix:
                    filename += '.tiff'

        elif Path(filename).suffix == '.png' or Path(filename).suffix == '.tiff':
            # Don't care if a file with the given name exists. Overwrite it if necessary.
            if path.exists(filename):
                remove(filename)
            # Write the image to the file. Before writing, convert the internal RGB representation into
            # the BGR representation assumed by OpenCV.
            if color:
                imwrite(str(filename), cvtColor(image, COLOR_RGB2BGR))
            else:
                imwrite(str(filename), image)

        elif Path(filename).suffix == '.fits':
            # Flip image horizontally to preserve orientation
            image = flip(image, 0)
            if color:
                image = moveaxis(image, -1, 0)
            hdu = fits.PrimaryHDU(image)
            hdu.header['CREATOR'] = header
            hdu.writeto(filename, overwrite=True)

        else:
            raise TypeError("Attempt to write image format other than 'tiff' or 'fits'")

    @staticmethod
    def read_image(filename):
        """
        Read an image (in tiff, fits, png or jpg format) from a file.

        :param filename: Path name of the input image.
        :return: RGB or monochrome image.
        """

        name, suffix = splitext(filename)

        # Make sure files with extensions written in large print can be read as well.
        suffix = suffix.lower()

        # Case FITS format:
        if suffix in ('.fit', '.fits'):
            image = fits.getdata(filename)

            # FITS output file from AS3 is 16bit depth file, even though BITPIX
            # has been set to "-32", which would suggest "numpy.float32"
            # https://docs.astropy.org/en/stable/io/fits/usage/image.html
            # To process this data in PSS, do "round()" and convert numpy array to "np.uint16"
            if image.dtype == '>f4':
                image = image.round().astype(uint16)

            # If color image, move axis to be able to process the content
            if len(image.shape) == 3:
                image = moveaxis(image, 0, -1).copy()

            # Flip image horizontally to recover original orientation
            image = flip(image, 0)

        # Case other supported image formats:
        elif suffix in ('.tiff', '.tif', '.png', '.jpg'):
            input_image = imread(filename, IMREAD_UNCHANGED)
            if input_image is None:
                raise IOError("Cannot read image file. Possible cause: Path contains non-ascii characters")

            # If color image, convert to RGB mode.
            if len(input_image.shape) == 3:
                image = cvtColor(input_image, COLOR_BGR2RGB)
            else:
                image = input_image

        else:
            raise TypeError("Attempt to read image format other than 'tiff', 'tif',"
                            " '.png', '.jpg' or 'fit', 'fits'")

        return image


if __name__ == "__main__":
    """
    This File contains a test main program. It goes through the whole process without using a
    graphical unser interface. It is not intended to be used in production runs.
    """

    ####################################### Specify test case ######################################

    input_file = 'E:/SW-Development/Python/PlanetarySystemStacker/Examples/Jupiter/2019-05-26-0115_4-L-Jupiter_ZWO ASI290MM Mini_short.avi'

    ####################################### Specify test case end ##################################

    # Initalize the timer object used to measure execution times of program sections.
    my_timer = timer()

    print(
        "\n" +
        "*************************************************************************************\n"
        + "Start processing " + str(
            input_file) +
        "\n*************************************************************************************")
    my_timer.create('Execution over all')

    # Get configuration parameters.
    configuration = Configuration()
    configuration.initialize_configuration()

    # Read the frames.
    print("+++ Start reading frames")
    my_timer.create('Read all frames')
    try:
        frames = Frames(configuration, input_file, type='video', buffer_original=True,
                        buffer_monochrome=False, buffer_gaussian=False, buffer_laplacian=False)
        print("Number of images read: " + str(frames.number))
        print("Image shape: " + str(frames.shape))
    except Error as e:
        print("Error: " + str(e))
        exit()
    my_timer.stop('Read all frames')

    # Rank the frames by their overall local contrast.
    print("+++ Start ranking images")
    my_timer.create('Ranking images')
    rank_frames = RankFrames(frames, configuration)

    configuration.frames_normalization = True
    rank_frames.frame_score()
    print ("Ranks with normalization: " + str(rank_frames.quality_sorted_indices))

    # Print out timer results.
    my_timer.stop('Execution over all')
    my_timer.print()

    # Initalize the timer object used to measure execution times of program sections.
    my_timer = timer()

    average_brightness_list = frames.frames_average_brightness

    # Now do the same without normalization.

    print(
        "\n" +
        "*************************************************************************************\n"
        + "Start processing " + str(
            input_file) +
        "\n*************************************************************************************")
    my_timer.create('Execution over all')

    # Get configuration parameters.
    configuration = Configuration()
    configuration.initialize_configuration()

    # Read the frames.
    print("+++ Start reading frames")
    my_timer.create('Read all frames')
    try:
        frames = Frames(configuration, input_file, type='video', buffer_original=True,
                        buffer_monochrome=False, buffer_gaussian=False, buffer_laplacian=False)
        print("Number of images read: " + str(frames.number))
        print("Image shape: " + str(frames.shape))
    except Error as e:
        print("Error: " + str(e))
        exit()
    my_timer.stop('Read all frames')

    # Rank the frames by their overall local contrast.
    print("+++ Start ranking images")
    my_timer.create('Ranking images')
    rank_frames = RankFrames(frames, configuration)

    configuration.frames_normalization = False
    rank_frames.frame_score_no_normalization(average_brightness_list)
    print("Ranks without normalization: " + str(rank_frames.quality_sorted_indices))

    # Print out timer results.
    my_timer.stop('Execution over all')
    my_timer.print()




# -*- coding: utf-8; -*-
"""
Copyright (c) 2019 Rolf Hempel, rolf6419@gmx.de

This file is part of the PlanetarySystemStacker tool (PSS).
https://github.com/Rolf-Hempel/PlanetarySystemStacker

PSS is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PSS.  If not, see <http://www.gnu.org/licenses/>.

"""

from PyQt5 import QtWidgets, QtCore
from PyQt5.QtGui import QIntValidator
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QLabel

from configuration import ConfigurationParameters
from parameter_configuration import Ui_ConfigurationDialog


class ConfigurationEditor(QtWidgets.QFrame, Ui_ConfigurationDialog):
    """
    Update the parameters used by PlanetarySystemStacker which are stored in the configuration
    object. The interaction with the user is through the ConfigurationDialog class.
    """

    def __init__(self, parent_gui, parent=None):
        QtWidgets.QFrame.__init__(self, parent)
        self.setupUi(self)

        self.setFrameShape(QtWidgets.QFrame.Panel)
        self.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.setObjectName("configuration_editor")

        self.setFixedSize(900, 600)

        self.parent_gui = parent_gui
        self.configuration = parent_gui.configuration

        # On return set the following variable to the activity to which the workflow has to go back
        # due to parameter changes. If it is None, nothing has to be repeated.
        self.configuration.go_back_to_activity = None
        self.configuration.configuration_changed = False

        # Create a ConfigurationParameters object and set it to the current parameters.
        self.config_copy = ConfigurationParameters()
        self.config_copy.copy_from_config_object(self.configuration)

        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        # Connect local methods with GUI change events.
        self.fgw_slider_value.valueChanged['int'].connect(self.fgw_changed)
        self.fdb_comboBox.addItem('Auto detect color')
        self.fdb_comboBox.addItem('Grayscale')
        self.fdb_comboBox.addItem('RGB')
        self.fdb_comboBox.addItem('BGR')
        self.fdb_comboBox.addItem('Force Bayer RGGB')
        self.fdb_comboBox.addItem('Force Bayer GRBG')
        self.fdb_comboBox.addItem('Force Bayer GBRG')
        self.fdb_comboBox.addItem('Force Bayer BGGR')
        self.fdb_comboBox.activated[str].connect(self.fdb_changed)
        self.fdbm_comboBox.addItem('Bilinear')
        self.fdbm_comboBox.addItem('Variable Number of Gradients')
        self.fdbm_comboBox.addItem('Edge Aware')
        self.fdbm_comboBox.activated[str].connect(self.fdbm_changed)
        self.fn_checkBox.stateChanged.connect(self.fn_changed)
        self.fnt_slider_value.valueChanged['int'].connect(self.fnt_changed)
        self.afm_comboBox.addItem('Surface')
        self.afm_comboBox.addItem('Planet')
        self.afm_comboBox.activated[str].connect(self.afm_changed)
        self.afa_checkBox.stateChanged.connect(self.afa_changed)
        self.afrsf_slider_value.valueChanged['int'].connect(self.afrsf_changed)
        self.afsw_slider_value.valueChanged['int'].connect(self.afsw_changed)
        self.afafp_slider_value.valueChanged['int'].connect(self.afafp_changed)
        self.efs_checkBox.stateChanged.connect(self.efs_changed)
        self.fco_checkBox.stateChanged.connect(self.fco_changed)
        self.gpwptf_checkBox.stateChanged.connect(self.gpwptf_changed)
        self.gpspwr_checkBox.stateChanged.connect(self.gpspwr_changed)
        self.gppl_spinBox.valueChanged['int'].connect(self.gppl_changed)
        self.gpbl_combobox.addItem('auto')
        self.gpbl_combobox.addItem('0')
        self.gpbl_combobox.addItem('1')
        self.gpbl_combobox.addItem('2')
        self.gpbl_combobox.addItem('3')
        self.gpbl_combobox.addItem('4')
        self.gpbl_combobox.activated[str].connect(self.gpbl_changed)
        self.mr_checkBox.stateChanged.connect(self.mr_changed)
        self.mr_lineEdit.setValidator(QIntValidator(1, 2147483647))
        self.mr_lineEdit.textChanged.connect(self.mr_text_changed)
        self.gpif_comboBox.addItem('png')
        self.gpif_comboBox.addItem('tiff')
        self.gpif_comboBox.addItem('fits')
        self.gpif_comboBox.activated[str].connect(self.gpif_changed)
        self.aphbw_slider_value.valueChanged['int'].connect(self.aphbw_changed)
        self.apsw_slider_value.valueChanged['int'].connect(self.apsw_changed)
        self.apst_slider_value.valueChanged['int'].connect(self.apst_changed)
        self.apbt_slider_value.valueChanged['int'].connect(self.apbt_changed)
        self.apfp_comboBox.addItem('Percent of frames to be stacked')
        self.apfp_comboBox.addItem('Number of frames to be stacked')
        self.apfp_comboBox.activated[str].connect(self.apfp_state_changed)
        self.apfp_spinBox.valueChanged['int'].connect(self.apfp_value_changed)
        self.spp_checkBox.stateChanged.connect(self.spp_changed)
        self.ipfn_checkBox.stateChanged.connect(self.ipfn_changed)
        self.nfs_checkBox.stateChanged.connect(self.nfs_changed)
        self.pfs_checkBox.stateChanged.connect(self.pfs_changed)
        self.apbs_checkBox.stateChanged.connect(self.apbs_changed)
        self.nap_checkBox.stateChanged.connect(self.nap_changed)
        self.sfdfs_comboBox.addItem('Off')
        self.sfdfs_comboBox.addItem('1.5x')
        self.sfdfs_comboBox.addItem('2x')
        self.sfdfs_comboBox.addItem('3x')
        self.sfdfs_comboBox.activated[str].connect(self.sfdfs_changed)

        self.restore_standard_values.clicked.connect(self.restore_standard_parameters)

        self.initialize_widgets_and_local_parameters()

    def initialize_widgets_and_local_parameters(self):
        """
        Initialize GUI widgets with current configuration parameter values.

        :return: -
        """

        self.fgw_slider_value.setValue(int((self.config_copy.frames_gauss_width + 1) / 2))
        self.fgw_label_display.setText(str(self.config_copy.frames_gauss_width))
        index = self.fdb_comboBox.findText(self.config_copy.frames_debayering_default,
                                           QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.fdb_comboBox.setCurrentIndex(index)
        index = self.fdbm_comboBox.findText(self.config_copy.frames_debayering_method,
                                            QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.fdbm_comboBox.setCurrentIndex(index)
        index = self.afm_comboBox.findText(self.config_copy.align_frames_mode,
                                           QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.afm_comboBox.setCurrentIndex(index)
        self.afm_activate_deactivate_widgets()
        self.afa_checkBox.setChecked(self.config_copy.align_frames_automation)
        self.afrsf_slider_value.setValue(
            int(100. / self.config_copy.align_frames_rectangle_scale_factor))
        self.afrsf_label_display.setText(
            str(int(100. / self.config_copy.align_frames_rectangle_scale_factor)))
        self.afsw_slider_value.setValue(self.config_copy.align_frames_search_width)
        self.afsw_label_display.setText(str(self.config_copy.align_frames_search_width))
        self.afafp_slider_value.setValue(self.config_copy.align_frames_average_frame_percent)
        self.afafp_label_display.setText(str(self.config_copy.align_frames_average_frame_percent))
        self.efs_checkBox.setChecked(self.config_copy.frames_add_selection_dialog)
        self.fco_checkBox.setChecked(self.config_copy.align_frames_fast_changing_object)
        self.gpwptf_checkBox.setChecked(self.config_copy.global_parameters_write_protocol_to_file)
        self.gpspwr_checkBox.setChecked(
            self.config_copy.global_parameters_store_protocol_with_result)
        self.gppl_spinBox.setValue(self.config_copy.global_parameters_protocol_level)
        if self.config_copy.global_parameters_buffering_level != -1:
            self.gpbl_combobox.setCurrentIndex(self.config_copy.global_parameters_buffering_level+1)
        else:
            self.gpbl_combobox.setCurrentIndex(0)

        self.mr_lineEdit.setText(str(self.config_copy.global_parameters_maximum_memory_amount))
        self.mr_checkBox.setChecked(self.config_copy.global_parameters_maximum_memory_active)
        self.make_mr_label_visible(self.config_copy.global_parameters_maximum_memory_active)
        self.mr_label.setStyleSheet('color: red')
        if not self.config_copy.global_parameters_maximum_memory_active:
            self.mr_lineEdit.setEnabled(False)

        index = self.gpif_comboBox.findText(self.config_copy.global_parameters_image_format,
                                           QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.gpif_comboBox.setCurrentIndex(index)
        self.ipfn_checkBox.setChecked(self.config_copy.global_parameters_parameters_in_filename)
        self.nfs_checkBox.setChecked(self.config_copy.global_parameters_stack_number_frames)
        self.pfs_checkBox.setChecked(self.config_copy.global_parameters_stack_percent_frames)
        self.apbs_checkBox.setChecked(self.config_copy.global_parameters_ap_box_size)
        self.nap_checkBox.setChecked(self.config_copy.global_parameters_ap_number)

        self.aphbw_slider_value.setValue(self.config_copy.alignment_points_half_box_width * 2)
        self.aphbw_label_display.setText(str(self.config_copy.alignment_points_half_box_width * 2))
        self.apsw_slider_value.setValue(self.config_copy.alignment_points_search_width)
        self.apsw_label_display.setText(str(self.config_copy.alignment_points_search_width))
        self.apst_slider_value.setValue(round(self.config_copy.alignment_points_structure_threshold * 100))
        self.apst_label_display.setText(str(self.config_copy.alignment_points_structure_threshold))
        self.apbt_slider_value.setValue(self.config_copy.alignment_points_brightness_threshold)
        self.apbt_label_display.setText(str(self.config_copy.alignment_points_brightness_threshold))
        if self.config_copy.alignment_points_frame_percent != -1:
            self.apfp_comboBox.setCurrentIndex(0)
            self.apfp_spinBox.setMaximum(100)
            self.apfp_spinBox.setValue(self.config_copy.alignment_points_frame_percent)
        else:
            self.apfp_comboBox.setCurrentIndex(1)
            self.apfp_spinBox.setMaximum(1000000)
            self.apfp_spinBox.setValue(self.config_copy.alignment_points_frame_number)
        self.spp_checkBox.setChecked(self.config_copy.global_parameters_include_postprocessing)
        self.fn_checkBox.setChecked(self.config_copy.frames_normalization)
        self.fn_activate_deactivate_widgets()
        self.fnt_slider_value.setValue(self.config_copy.frames_normalization_threshold)
        self.fnt_label_display.setText(str(self.config_copy.frames_normalization_threshold))
        self.ipfn_activate_deactivate_widgets()
        index = self.sfdfs_comboBox.findText(self.config_copy.stack_frames_drizzle_factor_string,
                                           QtCore.Qt.MatchFixedString)
        if index >= 0:
            self.sfdfs_comboBox.setCurrentIndex(index)

    def fgw_changed(self, value):
        """
        When the widget changes its value, update the corresponding entry in the configuration copy.
        Please note that for some parameters the representations differ.

        The methods following this one do the same for all other configuration parameters.

        :param value: New value sent by widget
        :return: -
        """

        self.config_copy.frames_gauss_width = 2 * value - 1
        self.fgw_label_display.setText(str(self.config_copy.frames_gauss_width))

    def fdb_changed(self, value):
        self.config_copy.frames_debayering_default = value

    def fdbm_changed(self, value):
        self.config_copy.frames_debayering_method = value

    def afm_changed(self, value):
        self.config_copy.align_frames_mode = value
        self.afm_activate_deactivate_widgets()

    def afm_activate_deactivate_widgets(self):
        """
        Special case: Depending on the "frame stabilization mode" selected, some other parameters
        do not make sense, so they are greyed out (for case 'Planet').

        :return:-
        """

        if self.config_copy.align_frames_mode == 'Planet':
            self.afa_checkBox.setEnabled(False)
            self.afrsf_label_display.setEnabled(False)
            self.afrsf_slider_value.setEnabled(False)
            self.afrsf_label_parameter.setEnabled(False)
            self.afsw_label_parameter.setEnabled(False)
            self.afsw_slider_value.setEnabled(False)
            self.afsw_label_display.setEnabled(False)
        else:
            self.afa_checkBox.setEnabled(True)
            self.afrsf_label_display.setEnabled(True)
            self.afrsf_slider_value.setEnabled(True)
            self.afrsf_label_parameter.setEnabled(True)
            self.afsw_label_parameter.setEnabled(True)
            self.afsw_slider_value.setEnabled(True)
            self.afsw_label_display.setEnabled(True)

    def fn_activate_deactivate_widgets(self):
        if self.config_copy.frames_normalization:
            self.fnt_label_display.setEnabled(True)
            self.fnt_slider_value.setEnabled(True)
            self.fnt_label_parameter.setEnabled(True)
        else:
            self.fnt_label_display.setEnabled(False)
            self.fnt_slider_value.setEnabled(False)
            self.fnt_label_parameter.setEnabled(False)

    def ipfn_activate_deactivate_widgets(self):
        if self.config_copy.global_parameters_parameters_in_filename:
            self.nfs_checkBox.setEnabled(True)
            self.pfs_checkBox.setEnabled(True)
            self.apbs_checkBox.setEnabled(True)
            self.nap_checkBox.setEnabled(True)
        else:
            self.nfs_checkBox.setEnabled(False)
            self.pfs_checkBox.setEnabled(False)
            self.apbs_checkBox.setEnabled(False)
            self.nap_checkBox.setEnabled(False)

    def afa_changed(self, state):
        self.config_copy.align_frames_automation = (state == QtCore.Qt.Checked)

    def afrsf_changed(self, value):
        self.config_copy.align_frames_rectangle_scale_factor = 100. / value

    def afsw_changed(self, value):
        self.config_copy.align_frames_search_width = value

    def afafp_changed(self, value):
        self.config_copy.align_frames_average_frame_percent = value

    def efs_changed(self, state):
        self.config_copy.frames_add_selection_dialog = (state == QtCore.Qt.Checked)

    def fco_changed(self, state):
        self.config_copy.align_frames_fast_changing_object = (state == QtCore.Qt.Checked)

    def gpwptf_changed(self, state):
        self.config_copy.global_parameters_write_protocol_to_file = (state == QtCore.Qt.Checked)

    def gpspwr_changed(self, state):
        self.config_copy.global_parameters_store_protocol_with_result = (state == QtCore.Qt.Checked)

    def gppl_changed(self, value):
        self.config_copy.global_parameters_protocol_level = value

    def gpbl_changed(self, value):
        if value == "auto":
            self.config_copy.global_parameters_buffering_level = -1
        else:
            self.config_copy.global_parameters_buffering_level = int(value)

    def mr_activate(self, state):
        if state:
            self.config_copy.global_parameters_maximum_memory_active = True
            # With a fixed memory limit buffering is set to 'auto'
            self.gpbl_combobox.setCurrentText('auto')
            self.gpbl_combobox.setEnabled(False)
            self.mr_lineEdit.setEnabled(True)
        else:
            self.config_copy.global_parameters_maximum_memory_active = False
            self.config_copy.global_parameters_buffering_level = -1
            self.gpbl_combobox.setEnabled(True)
            self.mr_lineEdit.setEnabled(False)

    def make_mr_label_visible(self, state):
        if state:
            text = "PlanetarySystemStacker takes no responsibility for enough memory being " \
                   "available.\r\n" + "Setting the memory limit too high may cause degraded OS " \
                                      "performance and/or process crashes."
        else:
            text = ""
        self.mr_label.setText(text)

    def mr_changed(self, state):
        max_memory_active = (state == QtCore.Qt.Checked)
        self.mr_activate(max_memory_active)
        self.make_mr_label_visible(max_memory_active)

    def mr_text_changed(self, state):
        try:
            value = int(state)
            if value > 0:
                self.config_copy.global_parameters_maximum_memory_amount = value
            else:
                self.mr_lineEdit.setText(
                    str(self.config_copy.global_parameters_maximum_memory_amount))
                raise ValueError
        except ValueError:
            return

    def gpif_changed(self, value):
        self.config_copy.global_parameters_image_format = value

    def aphbw_changed(self, value):
        self.config_copy.alignment_points_half_box_width = int(value / 2)

    def apsw_changed(self, value):
        self.config_copy.alignment_points_search_width = value

    def apst_changed(self, value):
        self.config_copy.alignment_points_structure_threshold = value / 100.
        self.apst_label_display.setText(str(self.config_copy.alignment_points_structure_threshold))

    def apbt_changed(self, value):
        self.config_copy.alignment_points_brightness_threshold = value

    def apfp_state_changed(self, value):
        self.apfp_spinBox.blockSignals(True)
        if value == 'Percent of frames to be stacked':
            self.apfp_spinBox.setMaximum(100)
            if self.config_copy.alignment_points_frame_percent > 0:
                self.apfp_spinBox.setValue(self.config_copy.alignment_points_frame_percent)
            else:
                self.apfp_spinBox.setValue(0)
                self.apfp_spinBox.clear()
        else:
            self.apfp_spinBox.setMaximum(1000000)
            if self.config_copy.alignment_points_frame_number > 0:
                self.apfp_spinBox.setValue(self.config_copy.alignment_points_frame_number)
            else:
                self.apfp_spinBox.setValue(0)
                self.apfp_spinBox.clear()
        self.apfp_spinBox.blockSignals(False)

    def apfp_value_changed(self, value):
        if self.apfp_comboBox.currentIndex() == 0:
            self.config_copy.alignment_points_frame_percent = value
        else:
            self.config_copy.alignment_points_frame_number = value

    def spp_changed(self, state):
        self.config_copy.global_parameters_include_postprocessing = (state == QtCore.Qt.Checked)

    def fn_changed(self, state):
        self.config_copy.frames_normalization = (state == QtCore.Qt.Checked)
        self.fn_activate_deactivate_widgets()

    def fnt_changed(self, value):
        self.config_copy.frames_normalization_threshold = value
        self.fnt_label_display.setText(str(self.config_copy.frames_normalization_threshold))

    def ipfn_changed(self, state):
        self.config_copy.global_parameters_parameters_in_filename = (state == QtCore.Qt.Checked)
        self.ipfn_activate_deactivate_widgets()

    def nfs_changed(self, state):
        self.config_copy.global_parameters_stack_number_frames = (state == QtCore.Qt.Checked)

    def pfs_changed(self, state):
        self.config_copy.global_parameters_stack_percent_frames = (state == QtCore.Qt.Checked)

    def apbs_changed(self, state):
        self.config_copy.global_parameters_ap_box_size = (state == QtCore.Qt.Checked)

    def nap_changed(self, state):
        self.config_copy.global_parameters_ap_number = (state == QtCore.Qt.Checked)

    def sfdfs_changed(self, value):
        self.config_copy.stack_frames_drizzle_factor_string = value

    def restore_standard_parameters(self):
        """
        Reset configuration parameters and GUI widget settings to standard values. Mark
        configuration as changed. This may be too pessimistic, if standard values were not changed
        before.

        :return:
        """

        self.config_copy.set_defaults()
        self.initialize_widgets_and_local_parameters()

    def accept(self):
        """
        If the OK button is clicked and the configuration has been changed, check if values have
        been changed and assign the new values to configuration parameters.

        :return: -
        """

        go_back_to_activities = []

        if self.config_copy.alignment_points_frame_percent != \
                self.configuration.alignment_points_frame_percent or \
                self.config_copy.alignment_points_frame_number != \
                self.configuration.alignment_points_frame_number:
            # Additional to the state of the comboBox check if a valid number has been entered.
            if self.apfp_comboBox.currentIndex() == 0 and \
                    self.config_copy.alignment_points_frame_percent > 0:
                self.configuration.alignment_points_frame_percent = \
                    self.config_copy.alignment_points_frame_percent
                self.configuration.alignment_points_frame_number = -1
                self.configuration.configuration_changed = True
                go_back_to_activities.append('Select stack size')
            elif self.apfp_comboBox.currentIndex() == 1 and \
                    self.config_copy.alignment_points_frame_number > 0:
                self.configuration.alignment_points_frame_percent = -1
                self.configuration.alignment_points_frame_number = \
                    self.config_copy.alignment_points_frame_number
                self.configuration.configuration_changed = True
                go_back_to_activities.append('Select stack size')

        if self.config_copy.alignment_points_brightness_threshold != \
                self.configuration.alignment_points_brightness_threshold:
            self.configuration.alignment_points_brightness_threshold = \
                self.config_copy.alignment_points_brightness_threshold
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Set alignment points')

        if self.config_copy.alignment_points_structure_threshold != \
                self.configuration.alignment_points_structure_threshold:
            self.configuration.alignment_points_structure_threshold = \
                self.config_copy.alignment_points_structure_threshold
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Set alignment points')

        if self.config_copy.alignment_points_search_width != \
                self.configuration.alignment_points_search_width:
            self.configuration.alignment_points_search_width = \
                self.config_copy.alignment_points_search_width
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Set alignment points')

        if self.config_copy.alignment_points_half_box_width != \
                self.configuration.alignment_points_half_box_width:
            self.configuration.alignment_points_half_box_width = \
                self.config_copy.alignment_points_half_box_width
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Set alignment points')

        if self.config_copy.align_frames_average_frame_percent != \
                self.configuration.align_frames_average_frame_percent:
            self.configuration.align_frames_average_frame_percent = \
                self.config_copy.align_frames_average_frame_percent
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Align frames')

        if self.config_copy.align_frames_search_width != \
                self.configuration.align_frames_search_width:
            self.configuration.align_frames_search_width = \
                self.config_copy.align_frames_search_width
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Align frames')

        if self.config_copy.align_frames_rectangle_scale_factor != \
                self.configuration.align_frames_rectangle_scale_factor:
            self.configuration.align_frames_rectangle_scale_factor = \
                self.config_copy.align_frames_rectangle_scale_factor
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Align frames')

        if self.config_copy.align_frames_automation != self.configuration.align_frames_automation:
            self.configuration.align_frames_automation = self.config_copy.align_frames_automation
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Align frames')

        if self.config_copy.align_frames_mode != self.configuration.align_frames_mode:
            self.configuration.align_frames_mode = self.config_copy.align_frames_mode
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Align frames')

        if self.config_copy.frames_gauss_width != self.configuration.frames_gauss_width:
            self.configuration.frames_gauss_width = self.config_copy.frames_gauss_width
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Read frames')

        if self.config_copy.frames_debayering_default != self.configuration.frames_debayering_default:
            self.configuration.frames_debayering_default = self.config_copy.frames_debayering_default
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Read frames')

        if self.config_copy.frames_debayering_method != self.configuration.frames_debayering_method:
            self.configuration.frames_debayering_method = self.config_copy.frames_debayering_method
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Read frames')

        if self.config_copy.frames_normalization != self.configuration.frames_normalization:
            self.configuration.frames_normalization = self.config_copy.frames_normalization
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Compute frame qualities')

        if self.config_copy.frames_normalization_threshold != \
                self.configuration.frames_normalization_threshold:
            self.configuration.frames_normalization_threshold = \
                self.config_copy.frames_normalization_threshold
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Compute frame qualities')

        if self.config_copy.frames_add_selection_dialog != \
                self.configuration.frames_add_selection_dialog:
            self.configuration.frames_add_selection_dialog = \
                self.config_copy.frames_add_selection_dialog
            self.configuration.configuration_changed = True

        if self.config_copy.align_frames_fast_changing_object != \
                self.configuration.align_frames_fast_changing_object:
            self.configuration.align_frames_fast_changing_object = \
                self.config_copy.align_frames_fast_changing_object
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Align frames')

        if self.config_copy.global_parameters_store_protocol_with_result != \
                self.configuration.global_parameters_store_protocol_with_result:
            self.configuration.global_parameters_store_protocol_with_result = \
                self.config_copy.global_parameters_store_protocol_with_result
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_write_protocol_to_file != \
                self.configuration.global_parameters_write_protocol_to_file:
            self.configuration.global_parameters_write_protocol_to_file = \
                self.config_copy.global_parameters_write_protocol_to_file
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_protocol_level != \
                self.configuration.global_parameters_protocol_level:
            self.configuration.global_parameters_protocol_level = \
                self.config_copy.global_parameters_protocol_level
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_buffering_level != \
                self.configuration.global_parameters_buffering_level:
            self.configuration.global_parameters_buffering_level = \
                self.config_copy.global_parameters_buffering_level
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_maximum_memory_active != \
                self.configuration.global_parameters_maximum_memory_active:
            self.configuration.global_parameters_maximum_memory_active = \
                self.config_copy.global_parameters_maximum_memory_active
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_maximum_memory_amount != \
                self.configuration.global_parameters_maximum_memory_amount:
            self.configuration.global_parameters_maximum_memory_amount = \
                self.config_copy.global_parameters_maximum_memory_amount
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_include_postprocessing != \
                self.configuration.global_parameters_include_postprocessing:
            self.configuration.global_parameters_include_postprocessing = \
                self.config_copy.global_parameters_include_postprocessing
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_image_format != \
                self.configuration.global_parameters_image_format:
            self.configuration.global_parameters_image_format = \
                self.config_copy.global_parameters_image_format
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_parameters_in_filename != \
                self.configuration.global_parameters_parameters_in_filename:
            self.configuration.global_parameters_parameters_in_filename = \
                self.config_copy.global_parameters_parameters_in_filename
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_stack_number_frames != \
                self.configuration.global_parameters_stack_number_frames:
            self.configuration.global_parameters_stack_number_frames = \
                self.config_copy.global_parameters_stack_number_frames
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_stack_percent_frames != \
                self.configuration.global_parameters_stack_percent_frames:
            self.configuration.global_parameters_stack_percent_frames = \
                self.config_copy.global_parameters_stack_percent_frames
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_ap_box_size != \
                self.configuration.global_parameters_ap_box_size:
            self.configuration.global_parameters_ap_box_size = \
                self.config_copy.global_parameters_ap_box_size
            self.configuration.configuration_changed = True

        if self.config_copy.global_parameters_ap_number != \
                self.configuration.global_parameters_ap_number:
            self.configuration.global_parameters_ap_number = \
                self.config_copy.global_parameters_ap_number
            self.configuration.configuration_changed = True

        if self.config_copy.stack_frames_drizzle_factor_string != self.configuration.stack_frames_drizzle_factor_string:
            self.configuration.stack_frames_drizzle_factor_string = self.config_copy.stack_frames_drizzle_factor_string
            self.configuration.configuration_changed = True
            go_back_to_activities.append('Stack frames')

        # Set dependent parameters.
        self.configuration.set_derived_parameters()

        # If the change of parameters require going back in the workflow, find the latest phase
        # which is safe to go back to.
        if go_back_to_activities:
            self.parent_gui.signal_set_go_back_activity.emit(go_back_to_activities)

        self.close()

    def reject(self):
        """
        The Cancel button is pressed, discard the changes and close the GUI window.
        :return: -
        """

        self.configuration.configuration_changed = False
        self.configuration.go_back_to_activity = None
        self.close()

    def closeEvent(self, event):

        # Write ".ini" file if it does not exist yet or parameters have changed.
        if not self.configuration.config_file_exists or self.configuration.configuration_changed:
            self.configuration.write_config()

        self.parent_gui.display_widget(None, display=False)
        self.close()



# -*- coding: utf-8 -*-

from sys import argv
from urllib import request

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt

from gui_with_label import Ui_Form


class LabelGui(QtWidgets.QWidget, Ui_Form):
    def __init__(self):
        """
        Initialization of the widget.
        """
        super(LabelGui, self).__init__()
        self.setupUi(self)

        self.label.setTextFormat(Qt.MarkdownText)

        # md = self.get_markdown()

        md = """
# Demo

### Quickstart Guide

Hier Text

#### Configuration

Hier Text (es folgen hier drei Leerzeichen)
Mit Umbruch
"""

        self.label.setText(md)
        markdown_text = md.encode()
        print (str(markdown_text))

    def get_markdown(self):
        url = 'file:..\quickstart.md'
        response = request.urlopen(url)
        return response.read().decode()


if __name__ == '__main__':
    app = QtWidgets.QApplication(argv)
    window = LabelGui()
    window.show()
    app.exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'spinbox_gui.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(149, 74)
        self.spinBox = QtWidgets.QSpinBox(Frame)
        self.spinBox.setGeometry(QtCore.QRect(9, 27, 131, 20))
        self.spinBox.setObjectName("spinBox")




import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from cv2 import cvtColor, COLOR_RGB2GRAY, COLOR_BGR2RGB, imread, IMREAD_UNCHANGED
from numpy import ndarray
from numpy import uint8, uint16

from frames import debayer_frame


def apply_bayer(frame_in, bayer_pattern):
    """
    Convert a color (3D) frame into a 2D one by applying a Bayer pattern.

    :param frame_in: Color (three channel) image, either 8 or 16 bit unsigned int.
    :param bayer_pattern: Pattern used to convert the input image into the output image. One out
                          of 'Force Bayer RGGB', 'Force Bayer GRBG', 'Force Bayer GBRG',
                          'Force Bayer BGGR'
    :return: B/W (one channel) image with the same type and pixel dimensions as frame_in.
    """

    # Create the data structure of the output frame (one channel).
    pixel_y, pixel_x, dim = frame_in.shape
    frame_out = ndarray((pixel_y, pixel_x), dtype=frame_in.dtype)

    if dim != 3:
        raise Exception("Error: Input frame expected to be 3D (color)")

    # Apply the Bayer pattern. Just take the appropriate color channel at those places where
    # the pattern is set to a given color. There is no interpolation.
    if bayer_pattern == 'Force Bayer RGGB':
        channels = (0, 1, 1, 2)
    elif bayer_pattern == 'Force Bayer GRBG':
        channels = (1, 0, 2, 1)
    elif bayer_pattern == 'Force Bayer GBRG':
        channels = (1, 2, 0, 1)
    elif bayer_pattern == 'Force Bayer BGGR':
        channels = (2, 1, 1, 0)
    else:
        raise Exception("Invalid Bayer transformation: " + str(bayer_pattern))

    frame_out[0:pixel_y:2, 0:pixel_x:2] = frame_in[0:pixel_y:2, 0:pixel_x:2, channels[0]]
    frame_out[0:pixel_y:2, 1:pixel_x:2] = frame_in[0:pixel_y:2, 1:pixel_x:2, channels[1]]
    frame_out[1:pixel_y:2, 0:pixel_x:2] = frame_in[1:pixel_y:2, 0:pixel_x:2, channels[2]]
    frame_out[1:pixel_y:2, 1:pixel_x:2] = frame_in[1:pixel_y:2, 1:pixel_x:2, channels[3]]

    return frame_out


def show_image(frame, comment):
    """
    Use Matplotlib to show a given frame.

    :param frame: Color or B/W image.
    :param comment: Text string (to be displayed as window title)
    :return: -
    """
    print(comment + ", shape: " + str(frame.shape) + ", type: " + str(frame.dtype))

    # If image is 16bit, reduce it to 8bit for display.
    if frame.dtype == uint16:
        frame_shown = (frame >> 8).astype(uint8)
    else:
        frame_shown = frame

    plt.title(comment)
    if len(frame_shown.shape) == 3:
        plt.imshow(frame_shown)
    else:
        plt.imshow(frame_shown, cmap='gray')
    plt.show()


filename = 'Photodisc.png'
input_image = imread(filename, IMREAD_UNCHANGED)

DEPTH = 8

if input_image is None:
    raise IOError("Cannot read image file. Possible cause: Path contains non-ascii characters")

if DEPTH == 16:
    input_image = input_image.astype(uint16) << 8

# If color image, convert to RGB mode.
if len(input_image.shape) == 3:
    image_color = cvtColor(input_image, COLOR_BGR2RGB)
    image_grayscale = cvtColor(image_color, COLOR_RGB2GRAY)
else:
    image_grayscale = input_image
    pixel_y = input_image.shape[0]
    pixel_x = input_image.shape[1]
    image_color = ndarray((pixel_y, pixel_x, 3))
    image_color[:, :, 0] = input_image
    image_color[:, :, 1] = input_image
    image_color[:, :, 2] = input_image

show_image(image_color, "Color image read")

# Test Bayer encoding RGGB.
image_bayer_encoded = apply_bayer(image_color, 'Force Bayer RGGB')

show_image(image_bayer_encoded, "Image Bayer encoded RGGB")
image_bayer_decoded = debayer_frame(image_bayer_encoded, 'Force Bayer RGGB')
show_image(image_bayer_decoded, "Image Bayer decoded RGGB")

# Test Bayer encoding GRBG.
image_bayer_encoded = apply_bayer(image_color, 'Force Bayer GRBG')
image_bayer_decoded = debayer_frame(image_bayer_encoded, 'Force Bayer GRBG')
show_image(image_bayer_decoded, "Image Bayer decoded GRBG")

# Test Bayer encoding GBRG.
image_bayer_encoded = apply_bayer(image_color, 'Force Bayer GBRG')
image_bayer_decoded = debayer_frame(image_bayer_encoded, 'Force Bayer GBRG')
show_image(image_bayer_decoded, "Image Bayer decoded GBRG")

# Test Bayer encoding BGGR.
image_bayer_encoded = apply_bayer(image_color, 'Force Bayer BGGR')
image_bayer_decoded = debayer_frame(image_bayer_encoded, 'Force Bayer BGGR')
show_image(image_bayer_decoded, "Image Bayer decoded BGGR")

# Test "Force Grayscale" with color input.
image_bayer_decoded = debayer_frame(image_color, 'Grayscale')
show_image(image_bayer_decoded, "Image Grayscale")

# Test conversion to BGR with color input.
image_bgr = debayer_frame(image_color, 'BGR')
show_image(image_bgr, "Image BGR")
